@@ -98,6 +98,7 @@ from openmdao.visualization.timing_viewer.timing_viewer import view_timing, view
     view_MPI_timing
 from openmdao.visualization.options_widget import OptionsWidget
 from openmdao.visualization.case_viewer.case_viewer import CaseViewer
+from openmdao.visualization.tables.table_builder import generate_table
 
 # Notebook Utils
 from openmdao.utils.notebook_utils import notebook_mode, display_source, show_options_table, cite
@@ -117,8 +118,8 @@ from openmdao.utils.general_utils import wing_dbg, env_truthy
 from openmdao.utils.array_utils import shape_to_len
 
 # Reports System
-from openmdao.utils.reports_system import register_report, get_reports_dir, list_reports, \
-    clear_reports
+from openmdao.utils.reports_system import register_report, unregister_report, get_reports_dir, \
+    list_reports, clear_reports
 
 import os
 
