@@ -20,8 +20,8 @@ from openmdao.utils.options_dictionary import OptionsDictionary
 import openmdao.utils.coloring as coloring_mod
 from openmdao.utils.array_utils import sizes2offsets
 from openmdao.vectors.vector import _full_slice
-from openmdao.utils.indexer import indexer
-from openmdao.utils.om_warnings import issue_warning, DerivativesWarning
+from openmdao.utils.indexer import indexer, slicer
+from openmdao.utils.om_warnings import issue_warning, DerivativesWarning, DriverWarning
 import openmdao.utils.coloring as c_mod
 
 
@@ -115,6 +115,15 @@ class Driver(object):
                                   "iteration.",
                              default=[])
 
+        default_desvar_behavior = os.environ.get('OPENMDAO_INVALID_DESVAR_BEHAVIOR', 'warn').lower()
+
+        self.options.declare('invalid_desvar_behavior', values=('warn', 'raise', 'ignore'),
+                             desc='Behavior of driver if the initial value of a design '
+                                  'variable exceeds its bounds. The default value may be'
+                                  'set using the `OPENMDAO_INVALID_DESVAR_BEHAVIOR` environment '
+                                  'variable to one of the valid options.',
+                             default=default_desvar_behavior)
+
         # Case recording options
         self.recording_options = OptionsDictionary(parent_name=type(self).__name__)
 
@@ -149,6 +158,7 @@ class Driver(object):
 
         # What the driver supports.
         self.supports = OptionsDictionary(parent_name=type(self).__name__)
+        self.supports.declare('optimization', types=bool, default=False)
         self.supports.declare('inequality_constraints', types=bool, default=False)
         self.supports.declare('equality_constraints', types=bool, default=False)
         self.supports.declare('linear_constraints', types=bool, default=False)
@@ -160,7 +170,6 @@ class Driver(object):
         self.supports.declare('simultaneous_derivatives', types=bool, default=False)
         self.supports.declare('total_jac_sparsity', types=bool, default=False)
         self.supports.declare('distributed_design_vars', types=bool, default=True)
-        self.supports.declare('optimization', types=bool, default=True)
 
         self.iter_count = 0
         self.cite = ""
@@ -418,6 +427,47 @@ class Driver(object):
             msg = "Driver requires objective to be declared"
             raise RuntimeError(msg)
 
+    def _check_for_invalid_desvar_values(self):
+        """
+        Check for design variable values that exceed their bounds.
+
+        This method's behavior is controlled by the OPENMDAO_INVALID_DESVAR environment variable,
+        which may take on values 'ignore', 'error', 'warn'.
+        - 'ignore' : Proceed without checking desvar bounds.
+        - 'warn' : Issue a warning if one or more desvar values exceed bounds.
+        - 'raise' : Raise an exception if one or more desvar values exceed bounds.
+
+        These options are case insensitive.
+        """
+        if self.options['invalid_desvar_behavior'] != 'ignore':
+            invalid_desvar_data = []
+            for var, meta in self._designvars.items():
+                _val = self._problem().get_val(var, units=meta['units'], get_remote=True)
+                val = np.array([_val]) if np.ndim(_val) == 0 else _val  # Handle discrete desvars
+                idxs = meta['indices']() if meta['indices'] else None
+                flat_idxs = meta['flat_indices']
+                scaler = meta['scaler'] or 1.
+                adder = meta['adder'] or 0.
+                lower = meta['lower'] / scaler - adder
+                upper = meta['upper'] / scaler - adder
+                flat_val = val.ravel()[idxs] if flat_idxs else val[idxs].ravel()
+
+                if (flat_val < lower).any() or (flat_val > upper).any():
+                    invalid_desvar_data.append((var, val, lower, upper))
+            if invalid_desvar_data:
+                s = 'The following design variable initial conditions are out of their ' \
+                    'specified bounds:'
+                for var, val, lower, upper in invalid_desvar_data:
+                    s += f'\n  {var}\n    val: {val.ravel()}' \
+                         f'\n    lower: {lower}\n    upper: {upper}'
+                s += '\nSet the initial value of the design variable to a valid value or set ' \
+                     'the driver option[\'invalid_desvar_behavior\'] to \'ignore\'.'
+                s += '\nThis warning will become an error by default in OpenMDAO version 3.25.'
+                if self.options['invalid_desvar_behavior'] == 'raise':
+                    raise ValueError(s)
+                else:
+                    issue_warning(s, category=DriverWarning)
+
     def _get_vars_to_record(self, recording_options):
         """
         Get variables to record based on recording options.
@@ -702,7 +752,7 @@ class Driver(object):
                     # Setting an integer value with a 1D array - don't want to convert to array.
                     value = int(value)
                 else:
-                    value = value.astype(np.int)
+                    value = value.astype(int)
 
             problem.model._discrete_outputs[src_name] = value
 
