@@ -197,7 +197,7 @@ class SerialTests(unittest.TestCase):
         self.assertEqual(p.get_val('par.C2.y', get_remote=True), 'par.C2/')
 
     def test_discrete_fan_out2(self):
-        p = om.Problem()
+        p = om.Problem(name='discrete_fan_out2')
         model = p.model
         par = model.add_subsystem('par', om.ParallelGroup(), promotes=['x'])
         par.add_subsystem('C1', PathCompEx('foo'), promotes=['x'])
@@ -206,7 +206,12 @@ class SerialTests(unittest.TestCase):
         try:
             p.setup()
         except Exception as err:
-            self.assertEqual(str(err), "<model> <class Group>: The following inputs, ['par.C1.x', 'par.C2.x'], promoted to 'x', are connected but their metadata entries ['val'] differ. Call <group>.set_input_defaults('x', val=?), where <group> is the Group named 'par' to remove the ambiguity.")
+            self.assertEqual(str(err),
+              "\nCollected errors for problem 'discrete_fan_out2':"
+              "\n   <model> <class Group>: The following inputs, ['par.C1.x', 'par.C2.x'], promoted "
+              "to 'x', are connected but their metadata entries ['val'] differ. Call "
+              "<group>.set_input_defaults('x', val=?), where <group> is the Group named 'par' to "
+              "remove the ambiguity.")
         else:
             self.fail("Exception expected.")
 
