@@ -6,7 +6,7 @@ import numpy as np
 from io import StringIO
 
 import openmdao.api as om
-from openmdao.utils.assert_utils import assert_near_equal, assert_warning
+from openmdao.utils.assert_utils import assert_near_equal
 from openmdao.utils.mpi import MPI
 
 try:
@@ -248,7 +248,7 @@ class TestConnectionsPromoted(unittest.TestCase):
 
 class TestConnectionsIndices(unittest.TestCase):
 
-    def setUp(self):
+    def build_model(self, probname=None):
         class ArrayComp(om.ExplicitComponent):
             def setup(self):
                 self.add_input('inp', val=np.ones((2)))
@@ -262,20 +262,20 @@ class TestConnectionsIndices(unittest.TestCase):
         indep_var_comp.add_output('blammo', val=3.)
         indep_var_comp.add_output('arrout', val=np.ones(5))
 
-        prob = om.Problem()
+        prob = om.Problem(name=probname)
         prob.model.add_subsystem('idvp', indep_var_comp)
         prob.model.add_subsystem('arraycomp', ArrayComp())
 
         self.prob = prob
 
     def test_bad_shapes(self):
+        self.build_model('bad_shapes')
         # Should not be allowed because the source and target shapes do not match
         self.prob.model.connect('idvp.blammo', 'arraycomp.inp')
 
-        expected = "<model> <class Group>: The source and target shapes do not match or are " + \
+        expected = "\nCollected errors for problem 'bad_shapes':\n   <model> <class Group>: The source and target shapes do not match or are " + \
                    "ambiguous for the connection 'idvp.blammo' to 'arraycomp.inp'. " + \
                    "The source shape is (1,) but the target shape is (2,)."
-
         try:
             self.prob.setup()
         except Exception as err:
@@ -283,39 +283,16 @@ class TestConnectionsIndices(unittest.TestCase):
         else:
             self.fail('Exception expected.')
 
-        self.prob.model._raise_connection_errors = False
-
-        with assert_warning(UserWarning, expected):
-            self.prob.setup()
-
     def test_bad_length(self):
         # Should not be allowed because the length of src_indices is greater than
         # the shape of arraycomp.inp
+        self.build_model('bad_length')
         self.prob.model.connect('idvp.blammo', 'arraycomp.inp', src_indices=[0, 0, 0])
 
-        expected = "<model> <class Group>: The source indices [0 0 0] do not specify a valid shape " + \
+        expected = "\nCollected errors for problem 'bad_length':\n   <model> <class Group>: The source indices [0 0 0] do not specify a valid shape " + \
                    "for the connection 'idvp.blammo' to 'arraycomp.inp'. The target shape is " + \
                    "(2,) but indices are shape (3,)."
 
-        try:
-            self.prob.setup()
-        except ValueError as err:
-            self.assertEqual(str(err), expected)
-        else:
-            self.fail('Exception expected.')
-
-        self.prob.model._raise_connection_errors = False
-
-        with assert_warning(UserWarning, expected):
-            self.prob.setup()
-
-    def test_bad_value(self):
-        # Should not be allowed because the index value within src_indices is outside
-        # the valid range for the source
-        self.prob.model.connect('idvp.arrout', 'arraycomp.inp1', src_indices=[100000])
-
-        expected = "<model> <class Group>: When connecting 'idvp.arrout' to 'arraycomp.inp1': index 100000 is out of bounds for source dimension of size 5."
-
         try:
             self.prob.setup()
         except Exception as err:
@@ -323,30 +300,38 @@ class TestConnectionsIndices(unittest.TestCase):
         else:
             self.fail('Exception expected.')
 
-        self.prob.model._raise_connection_errors = False
+    def test_bad_value(self):
+        # Should not be allowed because the index value within src_indices is outside
+        # the valid range for the source
+        self.build_model('bad_value')
+        self.prob.model.connect('idvp.arrout', 'arraycomp.inp1', src_indices=[100000])
 
-        with assert_warning(om.SetupWarning, expected):
+        try:
             self.prob.setup()
+        except Exception as err:
+            self.assertEqual(str(err),
+               "\nCollected errors for problem 'bad_value':"
+               "\n   <model> <class Group>: When connecting 'idvp.arrout' to 'arraycomp.inp1': "
+               "index 100000 is out of bounds for source dimension of size 5.")
+        else:
+            self.fail('Exception expected.')
 
     def test_bad_value_bug(self):
         # Should not be allowed because the 2nd index value within src_indices is outside
         # the valid range for the source.  A bug prevented this from being checked.
+        self.build_model('bad_value_bug')
         self.prob.model.connect('idvp.arrout', 'arraycomp.inp', src_indices=[0, 100000])
 
-        expected = "<model> <class Group>: When connecting 'idvp.arrout' to 'arraycomp.inp': index 100000 is out of bounds for source dimension of size 5."
-
         try:
             self.prob.setup()
-        except IndexError as err:
-            self.assertEqual(str(err), expected)
+        except Exception as err:
+            self.assertEqual(str(err),
+               "\nCollected errors for problem 'bad_value_bug':"
+               "\n   <model> <class Group>: When connecting 'idvp.arrout' to 'arraycomp.inp': "
+               "index 100000 is out of bounds for source dimension of size 5.")
         else:
             self.fail('Exception expected.')
 
-        self.prob.model._raise_connection_errors = False
-
-        with assert_warning(UserWarning, expected):
-            self.prob.setup()
-
 
 class TestShapes(unittest.TestCase):
     def test_connect_flat_array_to_row_vector(self):
@@ -423,7 +408,7 @@ class TestShapes(unittest.TestCase):
                          5 * np.arange(10)[np.newaxis, np.newaxis, np.newaxis, :])
 
     def test_connect_incompatible_shapes(self):
-        p = om.Problem()
+        p = om.Problem(name='connect_incompatible_shapes')
         p.model.add_subsystem('indep', om.IndepVarComp('x', val=np.arange(10)[np.newaxis, :,
                                                                               np.newaxis, np.newaxis]))
         p.model.add_subsystem('C1', om.ExecComp('y=5*x',
@@ -431,7 +416,7 @@ class TestShapes(unittest.TestCase):
                                                 y={'val': np.zeros((5, 2))}))
         p.model.connect('indep.x', 'C1.x')
 
-        expected = "<model> <class Group>: The source and target shapes do not match or are " + \
+        expected = "\nCollected errors for problem 'connect_incompatible_shapes':\n   <model> <class Group>: The source and target shapes do not match or are " + \
                    "ambiguous for the connection 'indep.x' to 'C1.x'. The source shape is " + \
                    "(1, 10, 1, 1) but the target shape is (5, 2)."
 
@@ -440,11 +425,6 @@ class TestShapes(unittest.TestCase):
 
         self.assertEqual(str(context.exception), expected)
 
-        p.model._raise_connection_errors = False
-
-        with assert_warning(UserWarning, expected):
-            p.setup()
-
 
 class TestMultiConns(unittest.TestCase):
 
@@ -457,7 +437,7 @@ class TestMultiConns(unittest.TestCase):
                 self.add_subsystem('c2', om.ExecComp('z = 2*y', y=np.ones(4), z=2*np.ones(4)),
                                    promotes=['z', 'y'])
 
-        prob = om.Problem()
+        prob = om.Problem(name='mult_conns')
         indeps = prob.model.add_subsystem('indeps', om.IndepVarComp(), promotes=['*'])
         indeps.add_output('x', 10*np.ones(4))
         indeps.add_output('y', np.ones(4))
@@ -467,22 +447,17 @@ class TestMultiConns(unittest.TestCase):
         prob.model.connect('x', 'sub.x')
         prob.model.connect('y', 'sub.y')
 
-        expected = "<model> <class Group>: The following inputs have multiple connections: " + \
-                   "sub.c2.y from ['indeps.y', 'sub.c1.y']"
-
         with self.assertRaises(Exception) as context:
             prob.setup()
 
-        self.assertEqual(str(context.exception), expected)
-
-        prob.model._raise_connection_errors = False
-
-        with assert_warning(om.SetupWarning, expected):
-            prob.setup()
+        self.assertEqual(str(context.exception),
+           "\nCollected errors for problem 'mult_conns':"
+           "\n   <model> <class Group>: The following inputs have multiple connections: "
+                   "sub.c2.y from ['indeps.y', 'sub.c1.y'].")
 
     def test_mixed_conns_same_level(self):
 
-        prob = om.Problem()
+        prob = om.Problem(name='mixed_conns_same_level')
         indeps = prob.model.add_subsystem('indeps', om.IndepVarComp())
         indeps.add_output('x', 10*np.ones(4))
 
@@ -495,19 +470,15 @@ class TestMultiConns(unittest.TestCase):
         # make a second, explicit, connection to y (which is c2.y promoted)
         prob.model.connect('indeps.x', 'y')
 
-        expected = "<model> <class Group>: Input 'c2.y' cannot be connected to 'indeps.x' " + \
-                   "because it's already connected to 'c1.y'"
-
         with self.assertRaises(Exception) as context:
             prob.setup()
             prob.final_setup()
+            prob.run_model()
 
-        self.assertEqual(str(context.exception), expected)
-
-        prob.model._raise_connection_errors = False
-
-        with assert_warning(UserWarning, expected):
-            prob.setup()
+        self.assertEqual(str(context.exception),
+           "\nCollected errors for problem 'mixed_conns_same_level':"
+           "\n   <model> <class Group>: Input 'c2.y' cannot be connected to 'indeps.x' "
+                   "because it's already connected to 'c1.y'.")
 
     def test_auto_ivc_ambiguous_with_src_indices_msg(self):
 
@@ -526,7 +497,7 @@ class TestMultiConns(unittest.TestCase):
                 outputs['y'] = 2.0 * inputs['x']
 
 
-        prob = om.Problem()
+        prob = om.Problem(name='auto_ivc_ambiguous_with_src_indices_msg')
         model = prob.model
 
         prob.model.add_subsystem('c1', TComp(src_idx=[0, 1]), promotes_inputs=['x'])
@@ -534,17 +505,19 @@ class TestMultiConns(unittest.TestCase):
         prob.model.add_subsystem('d1', TComp(src_idx=[0, 1]), promotes_inputs=[('x', 'zz')])
         prob.model.add_subsystem('d2', TComp(src_idx=[1, 2]), promotes_inputs=[('x', 'zz')])
 
-        with self.assertRaises(RuntimeError) as context:
+        with self.assertRaises(Exception) as context:
             prob.setup()
 
-        msg = ("Attaching src_indices to inputs requires that the shape of the "
-               "source variable is known, but the source shape for inputs "
-               "['c1.x', 'c2.x'] is unknown. You can specify the src shape for "
-               "these inputs by setting 'val' or 'src_shape' in a call to "
-               "set_input_defaults, or by adding an IndepVarComp as the source.")
-
-        err_msg = str(context.exception).split(':')[-1]
-        self.assertEqual(err_msg, msg)
+        self.assertEqual(str(context.exception),
+           "\nCollected errors for problem 'auto_ivc_ambiguous_with_src_indices_msg':"
+           "\n   <model> <class Group>: Attaching src_indices to inputs requires that the shape "
+           "of the source variable is known, but the source shape for inputs ['c1.x', 'c2.x'] is "
+           "unknown. You can specify the src shape for these inputs by setting 'val' or 'src_shape' "
+           "in a call to set_input_defaults, or by adding an IndepVarComp as the source."
+           "\n   <model> <class Group>: Attaching src_indices to inputs requires that the shape of "
+           "the source variable is known, but the source shape for inputs ['d1.x', 'd2.x'] is "
+           "unknown. You can specify the src shape for these inputs by setting 'val' or 'src_shape' "
+           "in a call to set_input_defaults, or by adding an IndepVarComp as the source.")
 
 
 class TestAutoIVCAllowableShapeMismatch(unittest.TestCase):
@@ -620,17 +593,19 @@ class TestConnectionsDistrib(unittest.TestCase):
             def compute_partials(self, inputs, J):
                 J['y', 'x'] = np.ones((2,))
 
-        prob = om.Problem()
+        prob = om.Problem(name='serial_mpi_error')
         model = prob.model
         model.add_subsystem('p1', om.IndepVarComp('x', np.array([1.0, 3.0])))
         model.add_subsystem('c3', TestComp())
         model.connect("p1.x", "c3.x")
 
-        expected = f"Exception raised on rank 0: When accessing 'p1.x' with src_shape (2,) from 'c3.x' using src_indices [1 2]: index 2 is out of bounds for source dimension of size 2."
         try:
             prob.setup()
         except Exception as err:
-            self.assertEqual(str(err).splitlines()[-1], expected)
+            self.assertEqual(str(err),
+               "\nCollected errors for problem 'serial_mpi_error':"
+               "\n   'c3' <class TestComp>: When accessing 'p1.x' with src_shape (2,) from 'c3.x' "
+               "using src_indices [1 2]: index 2 is out of bounds for source dimension of size 2.")
         else:
             self.fail('Exception expected.')
 
@@ -649,18 +624,19 @@ class TestConnectionsDistrib(unittest.TestCase):
             def compute_partials(self, inputs, J):
                 J['y', 'x'] = np.ones((2,))
 
-        prob = om.Problem()
+        prob = om.Problem(name='serial_mpi_error_flat')
         model = prob.model
         model.add_subsystem('p1', om.IndepVarComp('x', np.array([1.0, 3.0])))
         model.add_subsystem('c3', TestComp())
         model.connect("p1.x", "c3.x")
 
-        expected = f"Exception raised on rank 0: When accessing 'p1.x' with src_shape (2,) from 'c3.x' using src_indices [1 2]: index 2 is out of bounds for source dimension of size 2."
-
         try:
             prob.setup()
         except Exception as err:
-            self.assertEqual(str(err).splitlines()[-1], expected)
+            self.assertEqual(str(err),
+               "\nCollected errors for problem 'serial_mpi_error_flat':"
+               "\n   'c3' <class TestComp>: When accessing 'p1.x' with src_shape (2,) from 'c3.x' "
+               "using src_indices [1 2]: index 2 is out of bounds for source dimension of size 2.")
         else:
             self.fail('Exception expected.')
 
@@ -693,7 +669,7 @@ class TestConnectionsError(unittest.TestCase):
             def compute_partials(self, inputs, J):
                 J['y', 'x'] = np.eye(2)
 
-        prob = om.Problem()
+        prob = om.Problem(name='incompatible_src_indices')
         model = prob.model
 
         rank = prob.comm.rank
@@ -712,7 +688,9 @@ class TestConnectionsError(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             prob.setup(check=False, mode='fwd')
         self.assertEqual(str(context.exception),
-                         "Exception raised on rank 0: When accessing 'p1.x' with src_shape (2,) from 'c3.x' using src_indices [1 2]: index 2 is out of bounds for source dimension of size 2.")
+            "\nCollected errors for problem 'incompatible_src_indices':"
+            "\n   'c3' <class TestComp>: When accessing 'p1.x' with src_shape (2,) from 'c3.x' "
+            "using src_indices [1 2]: index 2 is out of bounds for source dimension of size 2.")
 
 
 @unittest.skipUnless(MPI, "MPI is required.")
