@@ -1,7 +1,7 @@
 """ Unit tests for the Driver base class."""
 
 
-from distutils.version import LooseVersion
+from packaging.version import Version
 from io import StringIO
 import sys
 import unittest
@@ -156,9 +156,8 @@ class TestDriver(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
-        with assert_warning(OMDeprecationWarning, "'global_names' is deprecated in calls to _compute_totals. Use 'use_abs_names' instead."):
-            derivs = prob.driver._compute_totals(of=['comp.y1'], wrt=['px.x'], global_names=True,
-                                                 return_format='dict')
+        derivs = prob.driver._compute_totals(of=['comp.y1'], wrt=['px.x'], use_abs_names=True,
+                                             return_format='dict')
 
         oscale = np.array([1.0/(7.0-5.2), 1.0/(11.0-6.3)])
         iscale = np.array([2.0-0.5, 3.0-1.5])
@@ -364,7 +363,7 @@ class TestDriver(unittest.TestCase):
 
         try:
             # formatting has changed in numpy 1.14 and beyond.
-            if LooseVersion(np.__version__) >= LooseVersion("1.14"):
+            if Version(np.__version__) >= Version("1.14"):
                 with printoptions(precision=2, legacy="1.13"):
                     prob.run_driver()
             else:
@@ -416,7 +415,7 @@ class TestDriver(unittest.TestCase):
 
         try:
             # formatting has changed in numpy 1.14 and beyond.
-            if LooseVersion(np.__version__) >= LooseVersion("1.14"):
+            if Version(np.__version__) >= Version("1.14"):
                 with printoptions(precision=2, legacy="1.13"):
                     prob.run_driver()
             else:
@@ -448,7 +447,7 @@ class TestDriver(unittest.TestCase):
 
         try:
             # formatting has changed in numpy 1.14 and beyond.
-            if LooseVersion(np.__version__) >= LooseVersion("1.14"):
+            if Version(np.__version__) >= Version("1.14"):
                 with printoptions(precision=2, legacy="1.13"):
                     prob.run_driver()
             else:
@@ -481,13 +480,6 @@ class TestDriver(unittest.TestCase):
         msg = "Discrete design variables are not supported by this driver: p.xI"
         self.assertEqual(str(context.exception), msg)
 
-    def test_driver_recording_options_deprecated(self):
-        prob = om.Problem()
-        msg = "The recording option, record_model_metadata, on Driver is deprecated. " \
-              "Recording of model metadata will always be done"
-        with assert_warning(OMDeprecationWarning, msg):
-            prob.driver.recording_options['record_model_metadata'] = True
-
     def test_units_basic(self):
         prob = om.Problem()
         model = prob.model
