@@ -1,4 +1,5 @@
 """Simple example demonstrating how to implement an implicit component."""
+import sys
 import unittest
 
 from io import StringIO
@@ -7,6 +8,7 @@ import numpy as np
 
 import openmdao.api as om
 from openmdao.utils.assert_utils import assert_near_equal
+from openmdao.utils.general_utils import remove_whitespace
 from openmdao.test_suite.components.sellar import SellarImplicitDis1, SellarImplicitDis2
 
 
@@ -69,6 +71,9 @@ class QuadraticLinearize(QuadraticComp):
 
 class QuadraticJacVec(QuadraticComp):
 
+    def setup_partials(self):
+        pass  # prevent declaration of partials from base class
+
     def linearize(self, inputs, outputs, partials):
         a = inputs['a']
         b = inputs['b']
@@ -109,6 +114,56 @@ class QuadraticJacVec(QuadraticComp):
             d_residuals['x'] = self.inv_jac * d_outputs['x']
 
 
+class ImplCompTestCase(unittest.TestCase):
+
+    def test_add_input_output_retval(self):
+        # check basic metadata expected in return value
+        expected_ivp_input = {
+            'val': 3,
+            'shape': (1,),
+            'size': 1,
+            'units': 'ft',
+            'desc': '',
+            'tags': set(),
+        }
+        expected_ivp_output = {
+            'val': 3,
+            'shape': (1,),
+            'size': 1,
+            'units': 'ft',
+            'desc': '',
+            'tags': {'openmdao:allow_desvar'},
+        }
+        expected_discrete = {
+            'val': 3,
+            'type': int,
+            'desc': '',
+            'tags': set(),
+        }
+
+        class ImplComp(om.ImplicitComponent):
+            def setup(self):
+                meta = self.add_input('x', val=3.0, units='ft')
+                for key, val in expected_ivp_input.items():
+                    assert meta[key] == val, f'Expected {key}: {val} but got {key}: {meta[key]}'
+
+                meta = self.add_discrete_input('x_disc', val=3)
+                for key, val in expected_discrete.items():
+                    assert meta[key] == val, f'Expected {key}: {val} but got {key}: {meta[key]}'
+
+                meta = self.add_output('y', val=3.0, units='ft')
+                for key, val in expected_ivp_output.items():
+                    assert meta[key] == val, f'Expected {key}: {val} but got {key}: {meta[key]}'
+
+                meta = self.add_discrete_output('y_disc', val=3)
+                for key, val in expected_discrete.items():
+                    assert meta[key] == val, f'Expected {key}: {val} but got {key}: {meta[key]}'
+
+        prob = om.Problem()
+        prob.model.add_subsystem('comp', ImplComp())
+        prob.setup()
+
+
 class ImplicitCompTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -148,60 +203,60 @@ class ImplicitCompTestCase(unittest.TestCase):
         # cannot list_inputs on a Group before running
         model_inputs = self.prob.model.list_inputs(desc=True, out_stream=None)
         expected = {
-            'comp1.a': {'value': [1.], 'desc': ''},
-            'comp1.b': {'value': [1.], 'desc': ''},
-            'comp1.c': {'value': [1.], 'desc': ''},
-            'comp2.a': {'value': [1.], 'desc': ''},
-            'comp2.b': {'value': [1.], 'desc': ''},
-            'comp2.c': {'value': [1.], 'desc': ''},
+            'comp1.a': {'val': [1.], 'desc': ''},
+            'comp1.b': {'val': [1.], 'desc': ''},
+            'comp1.c': {'val': [1.], 'desc': ''},
+            'comp2.a': {'val': [1.], 'desc': ''},
+            'comp2.b': {'val': [1.], 'desc': ''},
+            'comp2.c': {'val': [1.], 'desc': ''},
         }
         self.assertEqual(dict(model_inputs), expected)
 
         # list_inputs on a component before running is okay
         c2_inputs = self.prob.model.comp2.list_inputs(desc=True, out_stream=None)
         expected = {
-            'a': {'value': [1.], 'desc': ''},
-            'b': {'value': [1.], 'desc': ''},
-            'c': {'value': [1.], 'desc': ''}
+            'a': {'val': [1.], 'desc': ''},
+            'b': {'val': [1.], 'desc': ''},
+            'c': {'val': [1.], 'desc': ''}
         }
         self.assertEqual(dict(c2_inputs), expected)
 
         # listing component inputs based on tags should work
         c2_inputs = self.prob.model.comp2.list_inputs(tags='tag_a', out_stream=None)
-        self.assertEqual(dict(c2_inputs), {'a': {'value': [1.]}})
+        self.assertEqual(dict(c2_inputs), {'a': {'val': [1.]}})
 
         # includes and excludes based on relative names should work
         c2_inputs = self.prob.model.comp2.list_inputs(includes='a', out_stream=None)
-        self.assertEqual(dict(c2_inputs), {'a': {'value': [1.]}})
+        self.assertEqual(dict(c2_inputs), {'a': {'val': [1.]}})
 
         c2_inputs = self.prob.model.comp2.list_inputs(excludes='c', out_stream=None)
         expected = {
-            'a': {'value': [1.]},
-            'b': {'value': [1.]},
+            'a': {'val': [1.]},
+            'b': {'val': [1.]},
         }
         self.assertEqual(dict(c2_inputs), expected)
 
         # specifying prom_name should not cause an error
         c2_inputs = self.prob.model.comp2.list_inputs(prom_name=True, out_stream=None)
         self.assertEqual(dict(c2_inputs), {
-            'a': {'value': [1.], 'prom_name': 'a'},
-            'b': {'value': [1.], 'prom_name': 'b'},
-            'c': {'value': [1.], 'prom_name': 'c'}
+            'a': {'val': [1.], 'prom_name': 'a'},
+            'b': {'val': [1.], 'prom_name': 'b'},
+            'c': {'val': [1.], 'prom_name': 'c'}
         })
 
     def test_list_outputs_before_run(self):
         # cannot list_outputs on a Group before running
         model_outputs = self.prob.model.list_outputs(out_stream=None)
         expected = {
-            'comp1.x': {'value': [0.]},
-            'comp2.x': {'value': [0.]},
+            'comp1.x': {'val': [0.]},
+            'comp2.x': {'val': [0.]},
         }
         self.assertEqual(dict(model_outputs), expected)
 
         # list_outputs on a component before running is okay
         c2_outputs = self.prob.model.comp2.list_outputs(out_stream=None)
         expected = {
-            'x': {'value': np.array([0.])}
+            'x': {'val': np.array([0.])}
         }
         self.assertEqual(dict(c2_outputs), expected)
 
@@ -216,15 +271,17 @@ class ImplicitCompTestCase(unittest.TestCase):
         c2_outputs = self.prob.model.comp2.list_outputs(excludes='x', out_stream=None)
         self.assertEqual(dict(c2_outputs), {})
 
-        # specifying residuals_tol should not cause an error. However this will be empty because
-        # of the residuals_tol
+        # specifying residuals_tol should not cause an error
+        # there are no residuals yet, so nothing should be filtered
         c2_outputs = self.prob.model.comp2.list_outputs(residuals_tol=.01, out_stream=None)
-        self.assertEqual(dict(c2_outputs), {})
+        self.assertEqual(dict(c2_outputs), {
+            'x': {'val': 0.}
+        })
 
         # specifying prom_name should not cause an error
         c2_outputs = self.prob.model.comp2.list_outputs(prom_name=True, out_stream=None)
         self.assertEqual(dict(c2_outputs), {
-            'x': {'value': 0., 'prom_name': 'x'}
+            'x': {'val': 0., 'prom_name': 'x'}
         })
 
     def test_list_inputs(self):
@@ -233,17 +290,17 @@ class ImplicitCompTestCase(unittest.TestCase):
         stream = StringIO()
         inputs = self.prob.model.list_inputs(hierarchical=False, desc=True, out_stream=stream)
         self.assertEqual(sorted(inputs), [
-            ('comp1.a', {'value':  [1.], 'desc': ''}),
-            ('comp1.b', {'value': [-4.], 'desc': ''}),
-            ('comp1.c', {'value':  [3.], 'desc': ''}),
-            ('comp2.a', {'value':  [1.], 'desc': ''}),
-            ('comp2.b', {'value': [-4.], 'desc': ''}),
-            ('comp2.c', {'value':  [3.], 'desc': ''})
+            ('comp1.a', {'val':  [1.], 'desc': ''}),
+            ('comp1.b', {'val': [-4.], 'desc': ''}),
+            ('comp1.c', {'val':  [3.], 'desc': ''}),
+            ('comp2.a', {'val':  [1.], 'desc': ''}),
+            ('comp2.b', {'val': [-4.], 'desc': ''}),
+            ('comp2.c', {'val':  [3.], 'desc': ''})
         ])
         text = stream.getvalue()
         self.assertEqual(text.count('comp1.'), 3)
         self.assertEqual(text.count('comp2.'), 3)
-        self.assertEqual(text.count('value'), 1)
+        self.assertEqual(text.count('val'), 1)
 
     def test_list_inputs_with_tags(self):
         self.prob.run_model()
@@ -285,7 +342,7 @@ class ImplicitCompTestCase(unittest.TestCase):
         self.assertEqual(text.count('  c  '), 4)
 
         num_non_empty_lines = sum([1 for s in text.splitlines() if s.strip()])
-        self.assertEqual(num_non_empty_lines, 12)
+        self.assertEqual(num_non_empty_lines, 11)
 
     def test_list_explicit_outputs(self):
         self.prob.run_model()
@@ -302,16 +359,16 @@ class ImplicitCompTestCase(unittest.TestCase):
         # No tags
         outputs = self.prob.model.list_outputs(explicit=False, hierarchical=False, out_stream=None)
         self.assertEqual(sorted(outputs), [
-            ('comp1.x', {'value': [3.]}),
-            ('comp2.x', {'value': [3.]}),
+            ('comp1.x', {'val': [3.]}),
+            ('comp2.x', {'val': [3.]}),
         ])
 
         # With tag
         outputs = self.prob.model.list_outputs(explicit=False, hierarchical=False, out_stream=None,
                                                tags="tag_x")
         self.assertEqual(sorted(outputs), [
-            ('comp1.x', {'value': [3.]}),
-            ('comp2.x', {'value': [3.]}),
+            ('comp1.x', {'val': [3.]}),
+            ('comp2.x', {'val': [3.]}),
         ])
 
         # Wrong tag
@@ -325,12 +382,12 @@ class ImplicitCompTestCase(unittest.TestCase):
         stream = StringIO()
         states = self.prob.model.list_outputs(explicit=False, residuals=True,
                                               hierarchical=False, out_stream=stream)
-        self.assertEqual([('comp1.x', {'value': [3.], 'resids': [0.]}),
-                          ('comp2.x', {'value': [3.], 'resids': [0.]})], sorted(states))
+        self.assertEqual([('comp1.x', {'val': [3.], 'resids': [0.]}),
+                          ('comp2.x', {'val': [3.], 'resids': [0.]})], sorted(states))
         text = stream.getvalue()
         self.assertEqual(1, text.count('comp1.x'))
         self.assertEqual(1, text.count('comp2.x'))
-        self.assertEqual(1, text.count('value'))
+        self.assertEqual(1, text.count('val'))
         self.assertEqual(1, text.count('resids'))
 
     def test_list_outputs_prom_name(self):
@@ -345,7 +402,7 @@ class ImplicitCompTestCase(unittest.TestCase):
         self.assertEqual(text.count('comp1.x'), 1)
         self.assertEqual(text.count('comp2.x'), 1)
         num_non_empty_lines = sum([1 for s in text.splitlines() if s.strip()])
-        self.assertEqual(num_non_empty_lines, 8)
+        self.assertEqual(num_non_empty_lines, 7)
 
     def test_list_residuals(self):
         self.prob.run_model()
@@ -362,7 +419,7 @@ class ImplicitCompTestCase(unittest.TestCase):
         self.assertEqual(text.count('comp1.x'), 1)
         self.assertEqual(text.count('comp2.x'), 1)
         self.assertEqual(text.count('varname'), 1)
-        self.assertEqual(text.count('value'), 0)
+        self.assertEqual(text.count('val'), 0)
         self.assertEqual(text.count('resids'), 1)
 
     def test_list_residuals_with_tol(self):
@@ -385,12 +442,35 @@ class ImplicitCompTestCase(unittest.TestCase):
 
         prob.run_model()
 
-        stdout = StringIO()
-        outputs = model.list_outputs(residuals_tol=0.01, residuals=True, out_stream=stdout)
-        text = stdout.getvalue().split('\n')
-        # P1 and D1 should not appear in the outputs section. This is being checked below
-        self.assertEqual(text[14], 'd2')
-        self.assertFalse('d1' in text)
+        # list outputs with residuals, p1 and d1 should not appear
+        sysout = sys.stdout
+        try:
+            stdout = StringIO()
+            sys.stdout = stdout
+            outputs = model.list_outputs(residuals_tol=0.01, residuals=True, out_stream=stdout)
+        finally:
+            sys.stdout = sysout
+
+        expected_text = [
+            "0 Explicit Output(s) in 'model'",
+            "",
+            "",
+            "1 Implicit Output(s) in 'model'",
+            "",
+            "varname  val           resids     ",
+            "-------  ------------  -----------",
+            "d2",
+            "  y2",  # values removed from comparison
+            "",
+            "",
+            ""
+        ]
+        captured_output = stdout.getvalue()
+
+        for i, line in enumerate(captured_output.split('\n')):
+            if line and not line.startswith('-'):
+                self.assertEqual(remove_whitespace(line.split('[')[0]),
+                                 remove_whitespace(expected_text[i]))
 
 
 class ImplicitCompGuessTestCase(unittest.TestCase):
@@ -472,7 +552,7 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
 
             def guess_nonlinear(self, inputs, outputs, resids):
 
-                if outputs._data.dtype == np.complex:
+                if outputs.asarray().dtype == complex:
                     raise RuntimeError('Vector should not be complex when guess_nonlinear is called.')
 
                 # Default initial state of zero for x takes us to x=1 solution.
@@ -488,7 +568,8 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
         indep.add_output('c', 3.0)
         model.add_subsystem('p', indep)
         model.add_subsystem('comp', ImpWithInitial())
-        model.add_subsystem('fn', om.ExecComp(['y = .03*a*x*x - .04*a*a*b*x - c']))
+        fn = model.add_subsystem('fn', om.ExecComp(['y = .03*a*x*x - .04*a*a*b*x - c']))
+        fn.declare_partials(of='*', wrt='*', method='cs')
 
         model.connect('p.a', 'comp.a')
         model.connect('p.a', 'fn.a')
@@ -748,8 +829,6 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
         assert_near_equal(prob['sub.comp2.y'], 77., 1e-5)
 
     def test_guess_nonlinear_feature(self):
-        import openmdao.api as om
-        import numpy as np
 
         class ImpWithInitial(om.ImplicitComponent):
             """
@@ -814,6 +893,10 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
                 # inputs is read_only, should not be allowed
                 inputs['x'] = 0.
 
+            def apply_nonlinear(self, inputs, outputs, residuals, discrete_inputs=None,
+                                discrete_outputs=None):
+                pass
+
         group = om.Group()
 
         group.add_subsystem('px', om.IndepVarComp('x', 77.0))
@@ -846,6 +929,10 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
             def guess_nonlinear(self, inputs, outputs, resids):
                 raise om.AnalysisError("It's just a scratch.")
 
+            def apply_nonlinear(self, inputs, outputs, residuals, discrete_inputs=None,
+                discrete_outputs=None):
+                pass
+
         group = om.Group()
 
         group.add_subsystem('px', om.IndepVarComp('x', 77.0))
@@ -878,6 +965,10 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
                 # inputs is read_only, should not be allowed
                 resids['y'] = 0.
 
+            def apply_nonlinear(self, inputs, outputs, residuals, discrete_inputs=None,
+                discrete_outputs=None):
+                pass
+            
         group = om.Group()
 
         group.add_subsystem('px', om.IndepVarComp('x', 77.0))
@@ -900,6 +991,39 @@ class ImplicitCompGuessTestCase(unittest.TestCase):
                          "'comp1' <class ImpWithInitial>: Attempt to set value of 'y' in residual vector "
                          "when it is read only.")
 
+    def test_apply_nonlinear_missing_override(self):
+        class ImpWithInitial(om.ImplicitComponent):
+
+            def setup(self):
+                self.add_input('x', 3.0)
+                self.add_output('y', 4.0)
+
+            def guess_nonlinear(self, inputs, outputs, resids):
+                # inputs is read_only, should not be allowed
+                inputs['x'] = 0.
+
+        group = om.Group()
+
+        group.add_subsystem('px', om.IndepVarComp('x', 77.0))
+        group.add_subsystem('comp1', ImpWithInitial())
+        group.add_subsystem('comp2', ImpWithInitial())
+        group.connect('px.x', 'comp1.x')
+        group.connect('comp1.y', 'comp2.x')
+
+        group.nonlinear_solver = om.NewtonSolver(solve_subsystems=False)
+        group.nonlinear_solver.options['maxiter'] = 1
+
+        prob = om.Problem(model=group)
+        prob.set_solver_print(level=0)
+        prob.setup()
+
+        with self.assertRaises(NotImplementedError) as cm:
+            prob.run_model()
+
+        self.assertEqual(str(cm.exception),
+                         "'comp1' <class ImpWithInitial>: Error calling apply_nonlinear(), "
+                         "ImplicitComponent.apply_nonlinear() must be overridden by the "
+                         "child class.")
 
 class ImplicitCompReadOnlyTestCase(unittest.TestCase):
 
@@ -1063,7 +1187,7 @@ class ImplicitCompReadOnlyTestCase(unittest.TestCase):
 
         # check input vector
         with self.assertRaises(ValueError) as cm:
-            prob.model.run_apply_linear(['linear'], 'fwd')
+            prob.model.run_apply_linear('fwd')
 
         self.assertEqual(str(cm.exception),
                          "'bad' <class BadComp>: Attempt to set value of 'a' in input vector "
@@ -1083,7 +1207,7 @@ class ImplicitCompReadOnlyTestCase(unittest.TestCase):
 
         # check input vector
         with self.assertRaises(ValueError) as cm:
-            prob.model.run_apply_linear(['linear'], 'fwd')
+            prob.model.run_apply_linear('fwd')
 
         self.assertEqual(str(cm.exception),
                          "'bad' <class BadComp>: Attempt to set value of 'x' in output vector "
@@ -1103,7 +1227,7 @@ class ImplicitCompReadOnlyTestCase(unittest.TestCase):
 
         # check input vector
         with self.assertRaises(ValueError) as cm:
-            prob.model.run_apply_linear(['linear'], 'fwd')
+            prob.model.run_apply_linear('fwd')
 
         self.assertEqual(str(cm.exception),
                          "'bad' <class BadComp>: Attempt to set value of 'a' in input vector "
@@ -1123,7 +1247,7 @@ class ImplicitCompReadOnlyTestCase(unittest.TestCase):
 
         # check input vector
         with self.assertRaises(ValueError) as cm:
-            prob.model.run_apply_linear(['linear'], 'fwd')
+            prob.model.run_apply_linear('fwd')
 
         self.assertEqual(str(cm.exception),
                          "'bad' <class BadComp>: Attempt to set value of 'x' in output vector "
@@ -1143,7 +1267,7 @@ class ImplicitCompReadOnlyTestCase(unittest.TestCase):
 
         # check input vector
         with self.assertRaises(ValueError) as cm:
-            prob.model.run_apply_linear(['linear'], 'rev')
+            prob.model.run_apply_linear('rev')
 
         self.assertEqual(str(cm.exception),
                          "'bad' <class BadComp>: Attempt to set value of 'x' in residual vector "
@@ -1162,7 +1286,7 @@ class ImplicitCompReadOnlyTestCase(unittest.TestCase):
         prob.run_model()
 
         with self.assertRaises(om.AnalysisError):
-            prob.model.run_apply_linear(['linear'], 'rev')
+            prob.model.run_apply_linear('rev')
 
         # verify read_only status is reset after AnalysisError
         prob['bad.a'] = 111.
@@ -1183,7 +1307,7 @@ class ImplicitCompReadOnlyTestCase(unittest.TestCase):
 
         # check input vector
         with self.assertRaises(ValueError) as cm:
-            prob.model.run_solve_linear(['linear'], 'rev')
+            prob.model.run_solve_linear('rev')
 
         self.assertEqual(str(cm.exception),
                          "'bad' <class BadComp>: Attempt to set value of 'x' in output vector "
@@ -1203,7 +1327,7 @@ class ImplicitCompReadOnlyTestCase(unittest.TestCase):
 
         # check input vector
         with self.assertRaises(ValueError) as cm:
-            prob.model.run_solve_linear(['linear'], 'fwd')
+            prob.model.run_solve_linear('fwd')
 
         self.assertEqual(str(cm.exception),
                          "'bad' <class BadComp>: Attempt to set value of 'x' in residual vector "
@@ -1222,7 +1346,7 @@ class ImplicitCompReadOnlyTestCase(unittest.TestCase):
         prob.model.run_linearize()
 
         with self.assertRaises(om.AnalysisError):
-            prob.model.run_solve_linear(['linear'], 'fwd')
+            prob.model.run_solve_linear('fwd')
 
         # verify read_only status is reset after AnalysisError
         prob.model.bad._vectors['residual']['linear']['x'] = 111.
@@ -1231,8 +1355,6 @@ class ImplicitCompReadOnlyTestCase(unittest.TestCase):
 class ListFeatureTestCase(unittest.TestCase):
 
     def setUp(self):
-        import openmdao.api as om
-        from openmdao.core.tests.test_impl_comp import QuadraticComp
 
         group = om.Group()
 
@@ -1272,12 +1394,12 @@ class ListFeatureTestCase(unittest.TestCase):
         # list inputs
         inputs = prob.model.list_inputs(out_stream=None)
         self.assertEqual(sorted(inputs), [
-            ('sub.comp1.a', {'value': [1.]}),
-            ('sub.comp1.b', {'value': [-4.]}),
-            ('sub.comp1.c', {'value': [3.]}),
-            ('sub.comp2.a', {'value': [1.]}),
-            ('sub.comp2.b', {'value': [-4.]}),
-            ('sub.comp2.c', {'value': [3.]})
+            ('sub.comp1.a', {'val': [1.]}),
+            ('sub.comp1.b', {'val': [-4.]}),
+            ('sub.comp1.c', {'val': [3.]}),
+            ('sub.comp2.a', {'val': [1.]}),
+            ('sub.comp2.b', {'val': [-4.]}),
+            ('sub.comp2.c', {'val': [3.]})
         ])
 
     def test_for_docs_list_no_values(self):
@@ -1309,42 +1431,6 @@ class ListFeatureTestCase(unittest.TestCase):
         ])
 
     def test_simple_list_vars_options(self):
-        import openmdao.api as om
-
-        class QuadraticComp(om.ImplicitComponent):
-            """
-            A Simple Implicit Component representing a Quadratic Equation.
-
-            R(a, b, c, x) = ax^2 + bx + c
-
-            Solution via Quadratic Formula:
-            x = (-b + sqrt(b^2 - 4ac)) / 2a
-            """
-
-            def setup(self):
-                self.add_input('a', val=1., units='ft')
-                self.add_input('b', val=1., units='inch')
-                self.add_input('c', val=1., units='ft')
-                self.add_output('x', val=0.,
-                                lower=1.0, upper=100.0,
-                                ref=1.1, ref0=2.1,
-                                units='inch')
-
-            def setup_partials(self):
-                self.declare_partials(of='*', wrt='*')
-
-            def apply_nonlinear(self, inputs, outputs, residuals):
-                a = inputs['a']
-                b = inputs['b']
-                c = inputs['c']
-                x = outputs['x']
-                residuals['x'] = a * x ** 2 + b * x + c
-
-            def solve_nonlinear(self, inputs, outputs):
-                a = inputs['a']
-                b = inputs['b']
-                c = inputs['c']
-                outputs['x'] = (-b + (b ** 2 - 4 * a * c) ** 0.5) / (2 * a)
 
         group = om.Group()
 
@@ -1390,28 +1476,26 @@ class ListFeatureTestCase(unittest.TestCase):
         self.assertEqual(1, text.count("\n  comp2"))
         self.assertEqual(2, text.count("\n    a"))
         num_non_empty_lines = sum([1 for s in text.splitlines() if s.strip()])
-        self.assertEqual(num_non_empty_lines, 13)
+        self.assertEqual(num_non_empty_lines, 12)
 
         # list_outputs tests
         # list implicit outputs
         outputs = prob.model.list_outputs(explicit=False, out_stream=None)
         text = stream.getvalue()
         self.assertEqual(sorted(outputs), [
-            ('sub.comp2.x', {'value': [3.]}),
-            ('sub.comp3.x', {'value': [3.]})
+            ('sub.comp2.x', {'val': [3.]}),
+            ('sub.comp3.x', {'val': [3.]})
         ])
         # list explicit outputs
         stream = StringIO()
         outputs = prob.model.list_outputs(implicit=False, out_stream=None)
         self.assertEqual(sorted(outputs), [
-            ('comp1.a', {'value': [1.]}),
-            ('comp1.b', {'value': [-4.]}),
-            ('comp1.c', {'value': [3.]}),
+            ('comp1.a', {'val': [1.]}),
+            ('comp1.b', {'val': [-4.]}),
+            ('comp1.c', {'val': [3.]}),
         ])
 
     def test_list_residuals_with_tol(self):
-        import openmdao.api as om
-        from openmdao.test_suite.components.sellar import SellarImplicitDis1, SellarImplicitDis2
         prob = om.Problem()
         model = prob.model
 
@@ -1432,7 +1516,6 @@ class ListFeatureTestCase(unittest.TestCase):
         prob.run_model()
 
         outputs = model.list_outputs(residuals_tol=0.01, residuals=True)
-        print(outputs)
 
 
 class CacheUsingComp(om.ImplicitComponent):
