@@ -120,6 +120,7 @@ class ScipyOptimizeDriver(Driver):
         super().__init__(**kwargs)
 
         # What we support
+        self.supports['optimization'] = True
         self.supports['inequality_constraints'] = True
         self.supports['equality_constraints'] = True
         self.supports['two_sided_constraints'] = True
@@ -218,8 +219,8 @@ class ScipyOptimizeDriver(Driver):
             for name, meta in self._designvars.items():
                 lower = meta['lower']
                 upper = meta['upper']
-                if isinstance(lower, np.ndarray) or lower >= -INF_BOUND \
-                        or isinstance(upper, np.ndarray) or upper <= INF_BOUND:
+                if isinstance(lower, np.ndarray) or lower > -INF_BOUND \
+                        or isinstance(upper, np.ndarray) or upper < INF_BOUND:
                     self._cons[name] = meta.copy()
                     self._cons[name]['equals'] = None
                     self._cons[name]['linear'] = True
@@ -268,6 +269,7 @@ class ScipyOptimizeDriver(Driver):
         self._total_jac = None
 
         self._check_for_missing_objective()
+        self._check_for_invalid_desvar_values()
 
         # Initial Run
         with RecordingDebugging(self._get_name(), self.iter_count, self) as rec:
