@@ -19,6 +19,7 @@ from openmdao.test_suite.components.simple_comps import NonSquareArrayComp
 from openmdao.test_suite.groups.sin_fitter import SineFitter
 from openmdao.utils.assert_utils import assert_near_equal, assert_warning
 from openmdao.utils.general_utils import run_driver
+from openmdao.utils.testing_utils import set_env_vars_context
 from openmdao.utils.mpi import MPI
 
 try:
@@ -389,7 +390,7 @@ class TestScipyOptimizeDriver(unittest.TestCase):
 
         prob.driver = om.ScipyOptimizeDriver(optimizer='COBYLA', tol=1e-9, disp=False)
 
-        model.add_design_var('x', lower=-50.0, upper=50.0)
+        model.add_design_var('x')  # note: no bounds
         model.add_design_var('y', lower=-50.0, upper=50.0)
         model.add_objective('f_xy')
 
@@ -553,6 +554,113 @@ class TestScipyOptimizeDriver(unittest.TestCase):
 
         self.assertEqual(exception.args[0], msg)
 
+    def test_scipy_invalid_desvar_behavior(self):
+
+        expected_err = ("The following design variable initial conditions are out of their specified "
+                        "bounds:"
+                        "\n  paraboloid.x"
+                        "\n    val: [100.]"
+                        "\n    lower: -50.0"
+                        "\n    upper: 50.0"
+                        "\n  paraboloid.y"
+                        "\n    val: [-200.]"
+                        "\n    lower: -50.0"
+                        "\n    upper: 50.0"
+                        "\nSet the initial value of the design variable to a valid value or set "
+                        "the driver option['invalid_desvar_behavior'] to 'ignore'."
+                        "\nThis warning will become an error by default in OpenMDAO version 3.25.")
+
+        for option in ['warn', 'raise', 'ignore']:
+            with self.subTest(f'invalid_desvar_behavior = {option}'):
+                # build the model
+                prob = om.Problem()
+
+                prob.model.add_subsystem('paraboloid', om.ExecComp('f = (x-3)**2 + x*y + (y+4)**2 - 3'))
+
+                # setup the optimizationi
+                prob.driver = om.ScipyOptimizeDriver(invalid_desvar_behavior=option)
+                prob.driver.options['optimizer'] = 'SLSQP'
+
+                prob.model.add_design_var('paraboloid.x', lower=-50, upper=50)
+                prob.model.add_design_var('paraboloid.y', lower=-50, upper=50)
+                prob.model.add_objective('paraboloid.f')
+
+                prob.setup()
+
+                # Set initial values.
+                prob.set_val('paraboloid.x', 100.0)
+                prob.set_val('paraboloid.y', -200.0)
+
+                # run the optimization
+                if option == 'ignore':
+                    prob.run_driver()
+                elif option == 'raise':
+                    with self.assertRaises(ValueError) as ctx:
+                        prob.run_driver()
+                    self.assertEqual(str(ctx.exception), expected_err)
+                else:
+                    with assert_warning(om.DriverWarning, expected_err):
+                        prob.run_driver()
+
+                if option != 'raise':
+                    assert_near_equal(prob.get_val('paraboloid.x'), 6.66666666, tolerance=1.0E-5)
+                    assert_near_equal(prob.get_val('paraboloid.y'), -7.33333333, tolerance=1.0E-5)
+                    assert_near_equal(prob.get_val('paraboloid.f'), -27.33333333, tolerance=1.0E-5)
+
+    def test_scipy_invalid_desvar_behavior_env(self):
+
+        expected_err = ("The following design variable initial conditions are out of their specified "
+                        "bounds:"
+                        "\n  paraboloid.x"
+                        "\n    val: [100.]"
+                        "\n    lower: -50.0"
+                        "\n    upper: 50.0"
+                        "\n  paraboloid.y"
+                        "\n    val: [-200.]"
+                        "\n    lower: -50.0"
+                        "\n    upper: 50.0"
+                        "\nSet the initial value of the design variable to a valid value or set "
+                        "the driver option['invalid_desvar_behavior'] to 'ignore'."
+                        "\nThis warning will become an error by default in OpenMDAO version 3.25.")
+
+        for option in ['warn', 'raise', 'ignore']:
+            with self.subTest(f'invalid_desvar_behavior = {option}'):
+                with set_env_vars_context(OPENMDAO_INVALID_DESVAR_BEHAVIOR=option):
+                    # build the model
+                    prob = om.Problem()
+
+                    prob.model.add_subsystem('paraboloid', om.ExecComp('f = (x-3)**2 + x*y + (y+4)**2 - 3'))
+
+                    # setup the optimizationi
+                    prob.driver = om.ScipyOptimizeDriver()
+                    prob.driver.options['optimizer'] = 'SLSQP'
+
+                    prob.model.add_design_var('paraboloid.x', lower=-50, upper=50)
+                    prob.model.add_design_var('paraboloid.y', lower=-50, upper=50)
+                    prob.model.add_objective('paraboloid.f')
+
+                    prob.setup()
+
+                    # Set initial values.
+                    prob.set_val('paraboloid.x', 100.0)
+                    prob.set_val('paraboloid.y', -200.0)
+
+                    # run the optimization
+                    if option == 'ignore':
+                        prob.run_driver()
+                    elif option == 'raise':
+                        with self.assertRaises(ValueError) as ctx:
+                            prob.run_driver()
+                        self.assertEqual(str(ctx.exception), expected_err)
+                    else:
+                        with assert_warning(om.DriverWarning, expected_err):
+                            prob.run_driver()
+
+                    if option != 'raise':
+                        assert_near_equal(prob.get_val('paraboloid.x'), 6.66666666, tolerance=1.0E-5)
+                        assert_near_equal(prob.get_val('paraboloid.y'), -7.33333333, tolerance=1.0E-5)
+                        assert_near_equal(prob.get_val('paraboloid.f'), -27.33333333, tolerance=1.0E-5)
+
     def test_simple_paraboloid_double_sided_low(self):
 
         prob = om.Problem()
