@@ -17,7 +17,7 @@ from openmdao.utils.logger_utils import get_logger
 from openmdao.utils.class_util import overrides_method
 from openmdao.utils.mpi import MPI
 from openmdao.utils.hooks import _register_hook
-from openmdao.utils.general_utils import printoptions, ignore_errors
+from openmdao.utils.general_utils import printoptions
 from openmdao.utils.units import _has_val_mismatch
 from openmdao.utils.file_utils import _load_and_exec, text2html
 from openmdao.utils.om_warnings import issue_warning, SetupWarning
@@ -397,7 +397,7 @@ def _check_system_configs(problem, logger):
 
 def _check_solvers(problem, logger):
     """
-    Search over all solvers and raise an error for unsupported configurations.
+    Search over all solvers and warn about unsupported configurations.
 
     Report any implicit component that does not implement solve_nonlinear and
     solve_linear or have an iterative nonlinear and linear solver upstream of it.
@@ -765,7 +765,6 @@ def _check_config_cmd(options, user_args):
     _register_hook('final_setup', class_name='Problem', inst_id=options.problem, post=_check_config,
                    exit=True)
 
-    ignore_errors(True)
     _load_and_exec(options.file[0], user_args)
 
 
