@@ -87,6 +87,24 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
 
     linear_solver_class = om.DirectSolver
 
+    def test_specify_solver(self):
+
+        prob = om.Problem()
+        model = prob.model = SellarDerivatives()
+
+        model.nonlinear_solver = om.NonlinearBlockGS()
+        model.linear_solver = om.DirectSolver()
+
+        prob.setup()
+        prob.run_model()
+
+        wrt = ['z']
+        of = ['obj']
+
+        J = prob.compute_totals(of=of, wrt=wrt, return_format='flat_dict')
+        assert_near_equal(J['obj', 'z'][0][0], 9.61001056, .00001)
+        assert_near_equal(J['obj', 'z'][0][1], 1.78448534, .00001)
+
     # DirectSolver doesn't iterate.
     def test_solve_linear_maxiter(self):
         # Test that using options that should not exist in class cause an error
@@ -123,24 +141,24 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         # forward
         d_inputs, d_outputs, d_residuals = g1.get_linear_vectors()
 
-        d_residuals.set_const(1.0)
-        d_outputs.set_const(0.0)
+        d_residuals.set_val(1.0)
+        d_outputs.set_val(0.0)
         g1._linearize(g1._assembled_jac)
         g1.linear_solver._linearize()
-        g1.run_solve_linear(['linear'], 'fwd')
+        g1.run_solve_linear('fwd')
 
-        output = d_outputs._data
+        output = d_outputs.asarray()
         assert_near_equal(output, g1.expected_solution, 1e-15)
 
         # reverse
         d_inputs, d_outputs, d_residuals = g1.get_linear_vectors()
 
-        d_outputs.set_const(1.0)
-        d_residuals.set_const(0.0)
+        d_outputs.set_val(1.0)
+        d_residuals.set_val(0.0)
         g1.linear_solver._linearize()
-        g1.run_solve_linear(['linear'], 'rev')
+        g1.run_solve_linear('rev')
 
-        output = d_residuals._data
+        output = d_residuals.asarray()
         assert_near_equal(output, g1.expected_solution, 3e-15)
 
     def test_rev_mode_bug(self):
@@ -185,7 +203,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
 
         model.add_subsystem('indeps', om.IndepVarComp('x', np.arange(5).reshape((1,size,1))))
         model.add_subsystem('comp', om.ExecComp('y = x * 2.', x=np.zeros((size,)), y=np.zeros((size,))))
-        src_indices = [[0, i, 0] for i in range(size)]
+        src_indices = [[0]*size, np.arange(size, dtype=int), [0]*size]
         model.connect('indeps.x', 'comp.x', src_indices=src_indices)
 
         model.linear_solver = om.DirectSolver()
@@ -229,7 +247,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.run_model()
 
-        expected_msg = "Singular entry found in Group (thrust_equilibrium_group) for row associated with state/residual 'thrust' ('thrust_equilibrium_group.thrust_bal.thrust') index 0."
+        expected_msg = "Singular entry found in 'thrust_equilibrium_group' <class Group> for row associated with state/residual 'thrust' ('thrust_equilibrium_group.thrust_bal.thrust') index 0."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -245,7 +263,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(Exception) as cm:
             prob.setup()
 
-        expected_msg = "DupPartialsComp (dupcomp): d(x)/d(c): declare_partials has been called with rows and cols that specify the following duplicate subjacobian entries: [(4, 11), (10, 2)]."
+        expected_msg = "'dupcomp' <class DupPartialsComp>: d(x)/d(c): declare_partials has been called with rows and cols that specify the following duplicate subjacobian entries: [(4, 11), (10, 2)]."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -283,7 +301,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.run_model()
 
-        expected_msg = "Singular entry found in Group (thrust_equilibrium_group) for row associated with state/residual 'thrust' ('thrust_equilibrium_group.thrust_bal.thrust') index 0."
+        expected_msg = "Singular entry found in 'thrust_equilibrium_group' <class Group> for row associated with state/residual 'thrust' ('thrust_equilibrium_group.thrust_bal.thrust') index 0."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -320,7 +338,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.run_model()
 
-        expected_msg = "Singular entry found in Group (thrust_equilibrium_group) for row associated with state/residual 'thrust' ('thrust_equilibrium_group.thrust_bal.thrust') index 0."
+        expected_msg = "Singular entry found in 'thrust_equilibrium_group' <class Group> for row associated with state/residual 'thrust' ('thrust_equilibrium_group.thrust_bal.thrust') index 0."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -386,7 +404,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.compute_totals(of=['c5.y'], wrt=['p.x'])
 
-        expected_msg = "NaN entries found in Group (<model>) for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
+        expected_msg = "NaN entries found in <model> <class Group> for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -419,7 +437,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.compute_totals(of=['c5.y'], wrt=['p.x'])
 
-        expected_msg = "NaN entries found in Group (<model>) for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
+        expected_msg = "NaN entries found in <model> <class Group> for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -452,7 +470,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.compute_totals(of=['c5.y'], wrt=['p.x'])
 
-        expected_msg = "NaN entries found in Group (<model>) for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
+        expected_msg = "NaN entries found in <model> <class Group> for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -471,7 +489,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         model.connect('c1.y', 'c2.x')
         model.connect('c2.y', 'c3.x')
         model.connect('c3.y', 'c4.x')
-        model.connect('c4.y', 'c5.x', src_indices=([0]))
+        model.connect('c4.y', 'c5.x', src_indices=[0], flat_src_indices=True)
 
         model.linear_solver = om.DirectSolver(assemble_jac=True)
 
@@ -481,7 +499,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.compute_totals(of=['c5.y'], wrt=['p.x'])
 
-        expected_msg = "NaN entries found in Group (<model>) for rows associated with states/residuals ['c5.y']."
+        expected_msg = "NaN entries found in <model> <class Group> for rows associated with states/residuals ['c5.y']."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -500,7 +518,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         model.connect('c1.y', 'c2.x')
         model.connect('c2.y', 'c3.x')
         model.connect('c3.y', 'c4.x')
-        model.connect('c4.y', 'c5.x', src_indices=([0]))
+        model.connect('c4.y', 'c5.x', src_indices=([0]), flat_src_indices=True)
 
         model.linear_solver = om.DirectSolver(assemble_jac=True)
 
@@ -510,7 +528,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.compute_totals(of=['c5.y'], wrt=['p.x'])
 
-        expected_msg = "Singular entry found in Group (<model>) for row associated with state/residual 'c5.y' index 0."
+        expected_msg = "Singular entry found in <model> <class Group> for row associated with state/residual 'c5.y' index 0."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -529,7 +547,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         model.connect('c1.y', 'c2.x')
         model.connect('c2.y', 'c3.x')
         model.connect('c3.y', 'c4.x')
-        model.connect('c4.y', 'c5.x', src_indices=([0]))
+        model.connect('c4.y', 'c5.x', src_indices=[0], flat_src_indices=True)
 
         model.linear_solver = om.DirectSolver(assemble_jac=True)
         model.options['assembled_jac_type'] = 'dense'
@@ -540,7 +558,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.compute_totals(of=['c5.y'], wrt=['p.x'])
 
-        expected_msg = "Singular entry found in Group (<model>) for row associated with state/residual 'c5.y' index 0."
+        expected_msg = "Singular entry found in <model> <class Group> for row associated with state/residual 'c5.y' index 0."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -815,6 +833,59 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
 
         self.assertEqual(expected, str(cm.exception))
 
+    def test_error_msg_bad_svd(self):
+
+        class BadComp(om.ImplicitComponent):
+
+            def setup(self):
+
+                self.add_input('a', 3.0)
+
+                self.add_output('x', 1.0)
+                self.add_output('y', 1.0)
+                self.add_output('z', 1.0)
+
+                self.declare_partials('x', 'a', val=1.0)
+                self.declare_partials('x', 'a', val=1.0)
+                self.declare_partials('x', 'a', val=1.0)
+
+                self.declare_partials('x', 'x', val=-2.0)
+                self.declare_partials('x', 'y', val=-1.0e331)
+                self.declare_partials('x', 'z', val=-1.0e331)
+                self.declare_partials('y', 'x', val=-1.0)
+                self.declare_partials('y', 'y', val=-2.0)
+                self.declare_partials('y', 'z', val=-1.0e331)
+                self.declare_partials('z', 'x', val=-1.0e331)
+                self.declare_partials('z', 'y', val=-1.0)
+                self.declare_partials('z', 'z', val=-2.0)
+
+            def apply_nonlinear(self, inputs, outputs, residuals, discrete_inputs=None, discrete_outputs=None):
+                a = inputs['a']
+                x, y, z = outputs.values()
+
+                residuals['x'] = a - 2.0 * x - y - z
+                residuals['y'] = a - x - 2.0 * y - z
+                residuals['z'] = a - x - y - 2.0 * z
+
+        prob = om.Problem()
+        model = prob.model
+
+        model.add_subsystem('comp', BadComp(), promotes=['*'])
+
+        model.linear_solver = om.DirectSolver()
+
+        prob.setup()
+
+        prob.run_model()
+
+        with self.assertRaises(RuntimeError) as cm:
+            jac = prob.compute_totals(of=['x', 'y', 'z'], wrt=['a'])
+
+        expected = "Jacobian in '' is not full rank, but OpenMDAO was not " + \
+                    "able to determine which rows or columns."
+
+        self.assertEqual(expected, str(cm.exception))
+
     def test_matvec_error_raised(self):
         prob = om.Problem()
         model = prob.model
@@ -856,14 +927,14 @@ class TestDirectSolverRemoteErrors(unittest.TestCase):
         prob.model = group
         prob.model.linear_solver = om.DirectSolver()
         prob.model.connect('P.x', 'C1.x')
-        prob.model.connect('C1.y', 'C2.y')
+        prob.model.connect('C1.y', 'C2.y', src_indices=om.slicer[:])
 
 
         prob.setup(check=False, mode='fwd')
         with self.assertRaises(Exception) as cm:
             prob.run_model()
 
-        msg = "DirectSolver linear solver in Group (<model>) cannot be used in or above a ParallelGroup or a " + \
+        msg = "DirectSolver linear solver in <model> <class Group> cannot be used in or above a ParallelGroup or a " + \
             "distributed component."
         self.assertEqual(str(cm.exception), msg)
 
@@ -884,13 +955,13 @@ class TestDirectSolverRemoteErrors(unittest.TestCase):
 
         prob.model.linear_solver = om.DirectSolver()
         group.connect('P.x', 'sub.C1.x')
-        group.connect('sub.C1.y', 'sub.C2.y')
+        group.connect('sub.C1.y', 'sub.C2.y', src_indices=om.slicer[:])
 
         prob.setup(check=False, mode='fwd')
         with self.assertRaises(Exception) as cm:
             prob.run_model()
 
-        msg = "DirectSolver linear solver in Group (<model>) cannot be used in or above a ParallelGroup or a " + \
+        msg = "DirectSolver linear solver in <model> <class Group> cannot be used in or above a ParallelGroup or a " + \
             "distributed component."
         self.assertEqual(str(cm.exception), msg)
 
@@ -919,7 +990,7 @@ class TestDirectSolverRemoteErrors(unittest.TestCase):
         with self.assertRaises(Exception) as cm:
             prob.run_model()
 
-        msg = "DirectSolver linear solver in Group (<model>) cannot be used in or above a ParallelGroup or a " + \
+        msg = "DirectSolver linear solver in <model> <class Group> cannot be used in or above a ParallelGroup or a " + \
             "distributed component."
         self.assertEqual(str(cm.exception), msg)
 
@@ -940,35 +1011,11 @@ class TestDirectSolverRemoteErrors(unittest.TestCase):
         with self.assertRaises(Exception) as cm:
             prob.run_model()
 
-        msg = "DirectSolver linear solver in Group (<model>) cannot be used in or above a ParallelGroup or a " + \
+        msg = "DirectSolver linear solver in <model> <class Group> cannot be used in or above a ParallelGroup or a " + \
             "distributed component."
         self.assertEqual(str(cm.exception), msg)
 
 
-
-class TestDirectSolverFeature(unittest.TestCase):
-
-    def test_specify_solver(self):
-
-        import openmdao.api as om
-        from openmdao.test_suite.components.sellar import SellarDerivatives
-
-        prob = om.Problem()
-        model = prob.model = SellarDerivatives()
-
-        model.linear_solver = om.DirectSolver()
-
-        prob.setup()
-        prob.run_model()
-
-        wrt = ['z']
-        of = ['obj']
-
-        J = prob.compute_totals(of=of, wrt=wrt, return_format='flat_dict')
-        assert_near_equal(J['obj', 'z'][0][0], 9.61001056, .00001)
-        assert_near_equal(J['obj', 'z'][0][1], 1.78448534, .00001)
-
-
 class TestDirectSolverMPI(unittest.TestCase):
 
     N_PROCS = 2
