@@ -274,7 +274,13 @@ class Solver(object):
         depth : int
             depth of the current system (already incremented).
         """
-        self._system = weakref.ref(system)
+        if self._system is None:
+            self._system = weakref.ref(system)
+        elif self._system != weakref.ref(system):
+            raise RuntimeError(f"{type(self).__name__} has already been assigned to "
+                               f"{self._system().msginfo} and cannot also be assigned to "
+                               f"{system.msginfo}.")
+
         self._depth = depth
         self._problem_meta = system._problem_meta
 
