@@ -164,13 +164,18 @@ def assert_check_partials(data, atol=1e-6, rtol=1e-6):
         len_wrt_width = len(wrt_header)
         len_norm_width = len(norm_value_header)
         bad_derivs = []
+        inconsistent_derivs = set()
 
         # Find all derivatives whose errors exceed tolerance.
         # Also, size the output to precompute column extents.
-        for (var, wrt) in data[comp]:
-            pair_data = data[comp][var, wrt]
+        for key, pair_data in data[comp].items():
+            var, wrt = key
             for error_type, tolerance in [('abs error', atol), ('rel error', rtol), ]:
                 actual = pair_data[error_type]
+                incon = pair_data.get('rank_inconsistent')
+                if incon:
+                    inconsistent_derivs.add(key)
+
                 for error_val, mode in zip(actual, norm_types):
                     in_error = False
 
@@ -182,8 +187,8 @@ def assert_check_partials(data, atol=1e-6, rtol=1e-6):
                         if not np.allclose(error_val, 0.0, atol=tolerance):
 
                             if error_type == 'rel error' and mode == 'fwd-fd' and \
-                               np.allclose(pair_data['J_fwd'], 0.0, atol=atol) and \
-                               np.allclose(pair_data['J_fd'], 0.0, atol=atol):
+                                    np.allclose(pair_data['J_fwd'], 0.0, atol=atol) and \
+                                    np.allclose(pair_data['J_fd'], 0.0, atol=atol):
                                 # Special case: both fd and fwd are really tiny, so we want to
                                 # ignore the rather large relative errors.
                                 in_error = False
@@ -196,21 +201,28 @@ def assert_check_partials(data, atol=1e-6, rtol=1e-6):
                         in_error = True
 
                     if in_error:
-                        wrt_string = '{0} wrt {1}'.format(var, wrt)
-                        norm_string = '{}'.format(error_val)
+                        wrt_string = f'{var} wrt {wrt}'
+                        norm_string = str(error_val)
                         bad_derivs.append((wrt_string, norm_string, error_type, mode))
                         len_wrt_width = max(len_wrt_width, len(wrt_string))
                         len_norm_width = max(len_norm_width, len(norm_string))
 
-        if bad_derivs:
+        if bad_derivs or inconsistent_derivs:
             comp_error_string = ''
-            for wrt_string, norm_string, error_type, mode in bad_derivs:
-                err_msg = '{0} | {1} | {2} | {3}'.format(
-                    pad_name(wrt_string, len_wrt_width),
-                    pad_name(error_type.split()[0], len_absrel_width),
-                    pad_name(mode, len_norm_type_width),
-                    pad_name(norm_string, len_norm_width)) + '\n'
-                comp_error_string += err_msg
+            if bad_derivs:
+                for wrt_string, norm_string, error_type, mode in bad_derivs:
+                    err_msg = '{0} | {1} | {2} | {3}'.format(
+                        pad_name(wrt_string, len_wrt_width),
+                        pad_name(error_type.split()[0], len_absrel_width),
+                        pad_name(mode, len_norm_type_width),
+                        pad_name(norm_string, len_norm_width)) + '\n'
+                    comp_error_string += err_msg
+
+            if inconsistent_derivs:
+                comp_error_string += (
+                    "\nInconsistent derivs across processes for keys: "
+                    f"{sorted(inconsistent_derivs)}.\nCheck that distributed outputs are properly "
+                    "reduced when computing\nderivatives of serial inputs.")
 
             name_header = 'Component: {}\n'.format(comp)
             len_name_header = len(name_header)
@@ -235,7 +247,7 @@ def assert_check_partials(data, atol=1e-6, rtol=1e-6):
     # if error string then raise error with that string
     if error_string:
         header_line1 = 'Assert Check Partials failed for the following Components'
-        header_line2 = 'with absolute tolerance = {} and relative tolerance = {}'.format(atol, rtol)
+        header_line2 = f'with absolute tolerance = {atol} and relative tolerance = {rtol}'
         header_width = max(len(header_line1), len(header_line2))
         header = '\n' + header_width * '=' + '\n'
         header += header_line1 + '\n'
@@ -265,15 +277,19 @@ def assert_check_totals(totals_data, atol=1e-6, rtol=1e-6):
         Relative error. Default is 1e-6.
     """
     fails = []
-    for key, val in totals_data.items():
+    incon_keys = set()
+    for key, dct in totals_data.items():
+        if 'inconsistent_keys' in dct:
+            incon_keys = dct['inconsistent_keys']
+
         try:
-            val['J_fwd']
-            val['J_fd']
+            dct['J_fwd']
+            dct['J_fd']
         except Exception as err:
             raise err.__class__(f"For key {key}: {err}")
         try:
             for i in range(3):
-                erel, eabs = val['rel error'][i], val['abs error'][i]
+                erel, eabs = dct['rel error'][i], dct['abs error'][i]
                 if erel is not None and not np.isnan(erel):
                     if erel > rtol:
                         raise ValueError(f"rel tolerance of {erel} > allowed rel tolerance "
@@ -283,11 +299,23 @@ def assert_check_totals(totals_data, atol=1e-6, rtol=1e-6):
                         raise ValueError(f"abs tolerance of {eabs} > allowed abs tolerance "
                                          f"of {atol}.")
         except ValueError as err:
-            fails.append((key, val, err))
+            fails.append((key, dct, err))
+
+    fail_list = []
+
     if fails:
-        raise ValueError('\n\n'.join(
-            [f"Totals differ for {key}:\nAnalytic:\n{val['J_fwd']}\nFD:\n{val['J_fd']}\n{err}"
-             for key, val, err in fails]))
+        fail_list.extend(
+            [f"Totals differ for {key}:\nAnalytic:\n{dct['J_fwd']}\nFD:\n{dct['J_fd']}\n{err}"
+             for key, dct, err in fails])
+
+    if incon_keys:
+        ders = [f"{sof} wrt {swrt}" for sof, swrt in sorted(incon_keys)]
+        fail_list.append(f"During total derivative computation, the following partial derivatives "
+                         "resulted in serial inputs that were inconsistent across processes: "
+                         f"{ders}.")
+
+    if fails or incon_keys:
+        raise ValueError('\n\n'.join(fail_list))
 
 
 def assert_no_approx_partials(system, include_self=True, recurse=True, method='any', excludes=None):
@@ -459,6 +487,8 @@ def assert_near_equal(actual, desired, tolerance=1e-15):
     Determine that the relative error between `actual` and `desired`
     is within `tolerance`. If `desired` is zero, then use absolute error.
 
+    Can handle some data structures. Generates warnings for data types it cannot handle.
+
     Parameters
     ----------
     actual : float, array-like, dict
@@ -473,8 +503,48 @@ def assert_near_equal(actual, desired, tolerance=1e-15):
     float
         The error.
     """
-    if isinstance(actual, dict) and isinstance(desired, dict):
+    # Try to make similar things of the same type so they can be compared
+    # make arrays out of scalars
+    if type(actual) in [int, float, np.int64, np.float64, np.int32, np.complex128]:
+        actual = np.atleast_1d(actual)
+    if type(desired) in [int, float, np.int64, np.float64, np.int32, np.complex128]:
+        desired = np.atleast_1d(desired)
 
+    # if desired is numeric list or tuple, make ndarray out of it
+    if isinstance(actual, (list, tuple)):
+        actual = np.asarray(actual)
+    if isinstance(desired, (list, tuple)):
+        desired = np.asarray(desired)
+
+    # In case they are PromAbsDict and other dict-like objects
+    if isinstance(actual, dict) and type(actual) != dict:
+        actual = dict(actual)
+    if isinstance(desired, dict) and type(desired) != dict:
+        desired = dict(desired)
+
+    if type(actual) != type(desired):
+        raise ValueError('actual %s, desired %s have different types' % (actual, desired))
+
+    if isinstance(actual, type) and isinstance(desired, type):
+        if actual != desired:
+            raise ValueError(
+                'actual type %s, and desired type %s are different' % (actual, desired))
+        return 0
+
+    # The code below can only handle these data types
+    _supported_types = [dict, set, str, bool, np.ndarray, type(None)]
+    if type(actual) not in _supported_types:
+        warnings.warn(
+            f"The function, assert_near_equal, does not support the actual value type: '"
+            f"{type(actual)}'.")
+        return 0
+    if type(desired) not in _supported_types:
+        warnings.warn(
+            f"The function, assert_near_equal, does not support the desired value type: '"
+            f"{type(actual)}'.")
+        return 0
+
+    if isinstance(actual, dict) and isinstance(desired, dict):
         actual_keys = set(actual.keys())
         desired_keys = set(desired.keys())
 
@@ -498,44 +568,81 @@ def assert_near_equal(actual, desired, tolerance=1e-15):
                 msg = '{}: '.format(key) + str(exception)
                 raise KeyError(msg) from None
 
-    elif isinstance(actual, float) and isinstance(desired, float):
-        if isnan(actual) and not isnan(desired):
-            raise ValueError('actual nan, desired %s' % desired)
-        if desired != 0:
-            error = (actual - desired) / desired
-        else:
-            error = actual
-        if abs(error) > tolerance:
-            raise ValueError('actual %s, desired %s, rel error %s, tolerance %s'
-                             % (actual, desired, error, tolerance))
+    elif isinstance(actual, set) and isinstance(desired, set):
+        if actual.symmetric_difference(desired):
+            actual_extra = actual.difference(desired)
+            desired = desired.difference(actual)
+            raise KeyError("Actual and desired sets differ. "
+                           f"Actual extra values: {actual_extra}, "
+                           f"Desired extra values: {desired_extra}")
+
+        error = 0.
+
+    elif isinstance(actual, str) and isinstance(desired, str):
+        if actual != desired:
+            raise ValueError(
+                'actual %s, desired %s strings have different values' % (actual, desired))
+        error = 0.0
+
+    elif isinstance(actual, bool) and isinstance(desired, bool):
+        if actual != desired:
+            raise ValueError(
+                'actual %s, desired %s booleans have different values' % (actual, desired))
+        error = 0.0
+
+    elif actual is None and desired is None:
+        error = 0.0
 
     # array values
-    else:
-        actual = np.atleast_1d(actual)
-        desired = np.atleast_1d(desired)
-        if actual.shape != desired.shape:
-            raise ValueError(
-                'actual and desired have differing shapes.'
-                ' actual {}, desired {}'.format(actual.shape, desired.shape))
-        if not np.all(np.isnan(actual) == np.isnan(desired)):
-            if actual.size == 1 and desired.size == 1:
-                raise ValueError('actual %s, desired %s' % (actual, desired))
-            else:
-                raise ValueError('actual and desired values have non-matching nan'
-                                 ' values')
-
-        if np.linalg.norm(desired) == 0:
-            error = np.linalg.norm(actual)
+    elif isinstance(actual, np.ndarray) and isinstance(desired, np.ndarray):
+        if actual.dtype == object or desired.dtype == object:
+            if actual.dtype == object:
+                warnings.warn(
+                    f"The function, assert_near_equal, does not support the actual value ndarray "
+                    f"type of: '"
+                    f"{type(actual.dtype)}'.")
+            if desired.dtype == object:
+                warnings.warn(
+                    f"The function, assert_near_equal, does not support the desired value ndarray "
+                    f"type of: '"
+                    f"{type(desired.dtype)}'.")
+            error = 0.0
         else:
-            error = np.linalg.norm(actual - desired) / np.linalg.norm(desired)
 
-        if abs(error) > tolerance:
-            if actual.size < 10 and desired.size < 10:
-                raise ValueError('actual %s, desired %s, rel error %s, tolerance %s'
-                                 % (actual, desired, error, tolerance))
+            actual = np.atleast_1d(actual)
+            desired = np.atleast_1d(desired)
+            if actual.shape != desired.shape:
+                raise ValueError(
+                    'actual and desired have differing shapes.'
+                    ' actual {}, desired {}'.format(actual.shape, desired.shape))
+            # check to see if the entire array is made of floats. If not, loop through all values
+
+            if not np.all(np.isnan(actual) == np.isnan(desired)):
+                if actual.size == 1 and desired.size == 1:
+                    raise ValueError('actual %s, desired %s' % (actual, desired))
+                else:
+                    raise ValueError('actual and desired values have non-matching nan'
+                                     ' values')
+            if np.linalg.norm(desired) == 0:
+                error = np.linalg.norm(actual)
             else:
-                raise ValueError('arrays do not match, rel error %.3e > tol (%.3e)' %
-                                 (error, tolerance))
+                error = np.linalg.norm(actual - desired) / np.linalg.norm(desired)
+
+            if abs(error) > tolerance:
+                if actual.size < 10 and desired.size < 10:
+                    raise ValueError('actual %s, desired %s, rel error %s, tolerance %s'
+                                     % (actual, desired, error, tolerance))
+                else:
+                    raise ValueError('arrays do not match, rel error %.3e > tol (%.3e)' %
+                                     (error, tolerance))
+    elif isinstance(actual, tuple) and isinstance(desired, tuple):
+        error = 0.0
+        for act, des in zip(actual, desired):
+            new_error = assert_near_equal(act, des, tolerance)
+            error = max(error, new_error)
+    else:
+        raise ValueError(
+            'actual and desired have unexpected types: %s, %s' % (type(actual), type(desired)))
 
     return error
 
