@@ -162,7 +162,7 @@ class FakeComm(object):
 _om_mpi_debug = env_truthy('OM_MPI_DEBUG')
 
 
-def _debug_decorator(fn, scope):
+def _debug_decorator(fn, scope):  # pragma no cover
     def _wrap(*args, **kwargs):
         sc = '' if scope is None else f"{scope}."
         print(f"calling {sc}{fn.__name__}", flush=True)
@@ -172,7 +172,7 @@ def _debug_decorator(fn, scope):
     return _wrap
 
 
-class _DebugComm(object):
+class _DebugComm(object):  # pragma no cover
     """
     Debugging wrapper for an MPI communicator.
     """
@@ -194,20 +194,20 @@ class _DebugComm(object):
         setattr(self._comm, name, val)
 
 
-def _get_om_comm(comm, scope=None):
+def _get_om_comm(comm, scope=None):  # pragma no cover
     if _om_mpi_debug:
         return _DebugComm(comm, scope)
     return comm
 
 
-def _get_true_comm(comm):
+def _get_true_comm(comm):  # pragma no cover
     if isinstance(comm, _DebugComm):
         return comm._comm
     return comm
 
 
 @contextmanager
-def multi_proc_fail_check(comm):
+def multi_proc_fail_check(comm):  # pragma no cover
     """
     Raise an AnalysisError on all procs if it is raised on one.
 
@@ -243,7 +243,7 @@ def multi_proc_fail_check(comm):
 
 
 @contextmanager
-def multi_proc_exception_check(comm):
+def multi_proc_exception_check(comm):  # pragma no cover
     """
     Raise an exception on all procs if it is raised on one.
 
@@ -334,7 +334,7 @@ else:
 
 
 if MPI:
-    def mpirun_tests():
+    def mpirun_tests():  # pragma no cover
         """
         Run individual tests under MPI.
 
