@@ -12,11 +12,6 @@ except ImportError:
 
 from openmdao.utils.om_warnings import issue_warning
 
-try:
-    from tabulate import tabulate
-except ImportError:
-    tabulate = None
-
 colab = 'google.colab' in sys.modules
 
 
@@ -118,25 +113,13 @@ def show_options_table(reference, recording_options=False):
 
     if ipy:
         if not hasattr(obj, "options"):
-            html = obj.to_table(fmt='html')
+            opt = obj
         elif not recording_options:
-            html = obj.options.to_table(fmt='html')
+            opt = obj.options
         else:
-            html = obj.recording_options.to_table(fmt='html')
+            opt = obj.recording_options
 
-        # Jupyter notebook imposes right justification, so we have to enforce what we want:
-        # - Center table headers
-        # - Left justify table columns
-        # - Limit column width so there is adequate width left for the deprecation message
-        style = '<{tag} style="text-align:{align}; max-width:{width}; overflow-wrap:break-word;">'
-
-        cols = html.count('<th>')                 # there could be 5 or 6 columns
-        width = '300px' if cols > 5 else '600px'  # limit width depending on number of columns
-
-        html = html.replace('<th>', style.format(tag='th', align='center', width=width))
-        html = html.replace('<td>', style.format(tag='td', align='left', width=width))
-
-        return display(HTML(html))
+        return display(HTML(str(opt.to_table(fmt='html', display=False))))
     else:
         issue_warning("IPython is not installed. Run `pip install openmdao[notebooks]` or "
                       "`pip install openmdao[docs]` to upgrade.")
@@ -160,16 +143,13 @@ def cite(reference):
 
 def notebook_mode():
     """
-    Check if the environment is interactive and if tabulate is installed.
+    Check if the environment is interactive.
 
     Returns
     -------
     bool
         True if the environment is an interactive notebook.
     """
-    if ipy and tabulate is None:
-        issue_warning("Tabulate is not installed. Run `pip install openmdao[notebooks]` to "
-                      "install required dependencies. Using ASCII for outputs.")
     return ipy
 
 
