@@ -4,8 +4,8 @@ import numpy as np
 
 import openmdao.api as om
 from openmdao.utils.array_utils import evenly_distrib_idxs
-from openmdao.utils.assert_utils import assert_near_equal
-from openmdao.utils.mpi import MPI
+from openmdao.utils.assert_utils import assert_near_equal, assert_check_partials
+from openmdao.utils.mpi import MPI, multi_proc_exception_check
 
 try:
     from openmdao.parallel_api import PETScVector
@@ -224,9 +224,9 @@ class TestPETScVector3Proc(unittest.TestCase):
         model.add_subsystem('des_vars', comp)
 
         sub = model.add_subsystem('pp', om.ParallelGroup())
-        sub.add_subsystem('calc1', om.ExecComp('y = 2.0*x', x=np.ones((3, )), y=np.ones((3, ))))
-        sub.add_subsystem('calc2', om.ExecComp('y = 5.0*x', x=np.ones((3, )), y=np.ones((3, ))))
-        sub.add_subsystem('calc3', om.ExecComp('y = 7.0*x', x=np.ones((3, )), y=np.ones((3, ))))
+        c1 = sub.add_subsystem('calc1', om.ExecComp('y = 2.0*x', x=np.ones((3, )), y=np.ones((3, ))))
+        c2 = sub.add_subsystem('calc2', om.ExecComp('y = 5.0*x', x=np.ones((3, )), y=np.ones((3, ))))
+        c3 = sub.add_subsystem('calc3', om.ExecComp('y = 7.0*x', x=np.ones((3, )), y=np.ones((3, ))))
 
         model.connect('des_vars.v1', 'pp.calc1.x')
         model.connect('des_vars.v1', 'pp.calc2.x')
@@ -234,25 +234,33 @@ class TestPETScVector3Proc(unittest.TestCase):
 
         model.linear_solver = om.LinearBlockGS()
 
-        prob.setup()
+        with multi_proc_exception_check(prob.comm):
+            prob.setup()
 
-        prob.run_model()
+        with multi_proc_exception_check(prob.comm):
+            prob.run_model()
 
-        vec = prob.model._vectors['output']['nonlinear']
-        norm_val = vec.get_norm()
-        assert_near_equal(norm_val, 89.61584681293817, 1e-10)
+        with multi_proc_exception_check(prob.comm):
+            vec = prob.model._vectors['output']['nonlinear']
+            norm_val = vec.get_norm()
+            assert_near_equal(norm_val, 89.61584681293817, 1e-10)
 
-        J = prob.compute_totals(of=['pp.calc1.y', 'pp.calc2.y', 'pp.calc3.y'], wrt=['des_vars.v1'])
+        with multi_proc_exception_check(prob.comm):
+            J = prob.compute_totals(of=['pp.calc1.y', 'pp.calc2.y', 'pp.calc3.y'], wrt=['des_vars.v1'])
 
-        vec = prob.model._vectors['output']['linear']
-        norm_val = vec.get_norm()
-        assert_near_equal(norm_val, 8.888194417315589, 1e-10)
+        with multi_proc_exception_check(prob.comm):
+            vec = prob.model._vectors['output']['linear']
+            norm_val = vec.get_norm()
+            assert_near_equal(norm_val, 8.888194417315589, 1e-10)
 
         # test petsc dot while we're at it
-        vec.set_val(3.)
+        with multi_proc_exception_check(prob.comm):
+            vec.set_val(3.)
         vec2 = prob.model._vectors['residual']['linear']
-        vec2.set_val(4.)
-        assert_near_equal(vec.dot(vec2), 12.*13, 1e-10)
+        with multi_proc_exception_check(prob.comm):
+            vec2.set_val(4.)
+        with multi_proc_exception_check(prob.comm):
+            assert_near_equal(vec.dot(vec2), 12.*13, 1e-10)
 
 
 if __name__ == '__main__':
