@@ -22,14 +22,10 @@ except ImportError:
     mpl = None
 
 
-try:
-    from tabulate import tabulate
-except ImportError:
-    tabulate = None
-
 from openmdao.core.constants import INF_BOUND
 from openmdao.utils.mpi import MPI
 from openmdao.utils.om_warnings import issue_warning, DriverWarning
+from openmdao.visualization.tables.table_builder import generate_table
 
 
 # Report file constants
@@ -120,7 +116,7 @@ def opt_report(prob, outfile=None):
     driver = prob.driver
     if not driver.supports['optimization']:
         driver_class = type(driver).__name__
-        issue_warning(f"The optimizer report is not applicable for the {driver_class} Driver "
+        issue_warning(f"The optimizer report is not applicable for Driver type '{driver_class}', "
                       "which does not support optimization", category=DriverWarning)
         return
 
@@ -132,33 +128,6 @@ def opt_report(prob, outfile=None):
 
     outfilepath = pathlib.Path(prob.get_reports_dir(force=True)).joinpath(outfile)
 
-    if not tabulate:
-        s = '''
-<!DOCTYPE html>
-<html lang="en">
-<head>
-    <meta charset="UTF-8">
-    <title>Optimization Report - Unable to Generate</title>
-</head>
-<body>
-
-<p>The optimization report requires the tabulate package but it is not currently installed.</p>
-
-<p>To install the tablulate module:</p>
-
-<code>
-pip install tabulate
-</code>
-
-</body>
-</html>
-        '''
-        if create:
-            with open(outfilepath, 'w') as f:
-                f.write(s)
-
-        return
-
     driver_scaling = True
 
     # Collect data from the problem
@@ -267,7 +236,7 @@ def _make_header_table(prob):
     rows.append(['Wall clock run time:', runtime_formatted])
     rows.append(['Exit status:', prob.driver.opt_result['exit_status']])
 
-    return tabulate(rows, tablefmt='html')
+    return generate_table(rows, tablefmt='html')
 
 
 def _make_opt_value_table(driver):
@@ -288,8 +257,8 @@ def _make_opt_value_table(driver):
     for key, meta in driver.options.items():
         meta = driver.options._dict[key]
         opt_settings.append((key, meta['val'], meta['desc']))
-    opt_settings_table = tabulate(opt_settings, headers=['Setting', 'Val', 'Description'],
-                                  tablefmt='html')
+    opt_settings_table = generate_table(opt_settings, headers=['Setting', 'Val', 'Description'],
+                                        tablefmt='html')
 
     html = ''
     if opt_settings_table is not None:
@@ -345,8 +314,7 @@ def _make_obj_table(objs_meta, objs_vals,
                 row[col_name] = meta[col_name]
         rows.append(row)
 
-    return tabulate(rows, headers='keys', tablefmt='html', floatfmt='.4e', stralign='left',
-                    numalign='left')
+    return generate_table(rows, headers='keys', tablefmt='html')
 
 
 def _make_dvcons_table(meta_dict, vals_dict, kind,
@@ -408,8 +376,11 @@ def _make_dvcons_table(meta_dict, vals_dict, kind,
                 mean_val = np.mean(vals_dict[name])
                 row[col_name] = _indicate_value_is_derived_from_array(mean_val, vals_dict[name])
             elif col_name == 'min':
-                min_val = min(vals_dict[name])  # get min. Could be an array
-                min_val_as_str = _indicate_value_is_derived_from_array(min_val, vals_dict[name])
+                if isinstance(vals_dict[name], np.ndarray):
+                    min_val = min(vals_dict[name])  # get min. Could be an array
+                    min_val_as_str = _indicate_value_is_derived_from_array(min_val, vals_dict[name])
+                else:
+                    min_val_as_str = str(vals_dict[name])
                 comp = (vals_dict[name] - meta['lower']) < _bounds_tolerance
                 if np.any(comp):
                     row[col_name] = \
@@ -417,8 +388,11 @@ def _make_dvcons_table(meta_dict, vals_dict, kind,
                 else:
                     row[col_name] = min_val_as_str
             elif col_name == 'max':
-                max_val = max(vals_dict[name])
-                max_val_as_str = _indicate_value_is_derived_from_array(max_val, vals_dict[name])
+                if isinstance(vals_dict[name], np.ndarray):
+                    max_val = max(vals_dict[name])  # get max. Could be an array
+                    max_val_as_str = _indicate_value_is_derived_from_array(max_val, vals_dict[name])
+                else:
+                    max_val_as_str = str(vals_dict[name])
                 comp = (meta['upper'] - vals_dict[name]) < _bounds_tolerance
                 if np.any(comp):
                     row[col_name] = \
@@ -464,7 +438,7 @@ def _make_dvcons_table(meta_dict, vals_dict, kind,
                         '<span class="plot-unavailable">Visuals require matplotlib</span>'
             elif col_name == 'size':
                 row[col_name] = int(meta[col_name])  # sometimes size in the meta data is a numpy
-                # array so tabulate does different formatting for that
+                # array so generate_table does different formatting for that
             elif col_name in ['ref', 'ref0']:
                 if meta[col_name] is not None:
                     derived_val = np.mean(meta[col_name])
@@ -477,8 +451,7 @@ def _make_dvcons_table(meta_dict, vals_dict, kind,
 
         rows.append(row)
 
-    return tabulate(rows, headers='keys', tablefmt='unsafehtml', floatfmt='.4e', stralign='center',
-                    numalign='left')
+    return generate_table(rows, headers='keys', tablefmt='html', safe=False, precision='.4e')
 
 
 def _sparkline(kind, meta, val, width=300):
