@@ -23,12 +23,12 @@ from openmdao.visualization.opt_report.opt_report import opt_report, \
     _default_optimizer_report_filename
 
 try:
-    from tabulate import tabulate
+    import pyDOE2
 except ImportError:
-    tabulate = None
+    pyDOE2 = None
+
 
 @use_tempdirs
-@unittest.skipIf(tabulate is None, reason="package 'tabulate' is not installed")
 class TestOptimizationReport(unittest.TestCase):
 
     def setup_problem_and_run_driver(self, driver,
@@ -116,18 +116,18 @@ class TestOptimizationReport(unittest.TestCase):
         report_file_path = str(pathlib.Path(prob.get_reports_dir()).joinpath(_default_optimizer_report_filename))
 
         check_rows = {
-            # 'runtime': '<tr><td>Wall clock run time:',
-            'iter_count':  '<tr><td>Number of driver iterations:',
-            'obj_calls':   '<tr><td>Number of objective calls:',
-            'deriv_calls': '<tr><td>Number of derivative calls:',
-            'exit_status': '<tr><td>Exit status:'
+            # 'runtime': 'Wall clock run time:',
+            'iter_count':  'Number of driver iterations:',
+            'obj_calls':   'Number of objective calls:',
+            'deriv_calls': 'Number of derivative calls:',
+            'exit_status': 'Exit status:'
         }
 
         def second_cell(line):
             """
             get value from second cell of table row
             """
-            value = line.split('</td><td>')[1].split('</td>')[0].strip()
+            value = line.split('>', 4)[-1].split('<', 1)[0].strip()
             if value == '':
                 # The report will show an empty cell when the value is 'None'
                 value = None
@@ -142,10 +142,13 @@ class TestOptimizationReport(unittest.TestCase):
 
         with open(report_file_path, 'r') as f:
             for line in f.readlines():
-                if line.startswith('<tr><td>'):
-                    for key, row in check_rows.items():
-                        if line.startswith(row):
-                            reported[key] = second_cell(line)
+                line = line.lstrip()
+                if line.startswith('<tr'):
+                    parts = line.split('>', 1)
+                    if parts[1].startswith('<td'):
+                        for key, row in check_rows.items():
+                            if row in line:
+                                reported[key] = second_cell(line)
                 elif line.startswith('</table>'):
                     # skip the rest of the file
                     break
@@ -160,8 +163,14 @@ class TestOptimizationReport(unittest.TestCase):
                                           )
         expect = {'obj_calls': 0, 'deriv_calls': 0}
         self.check_opt_result(expected=expect)
-        opt_report(self.prob)
-        self.check_opt_report(expected=expect)
+
+        expected_warning_msg = "The optimizer report is not applicable for Driver type 'Driver', " \
+                               "which does not support optimization"
+        with assert_warning(DriverWarning, expected_warning_msg):
+            opt_report(self.prob)
+
+        outfilepath = str(pathlib.Path(self.prob.get_reports_dir()).joinpath(_default_optimizer_report_filename))
+        self.assertFalse(os.path.exists(outfilepath))
 
     def test_opt_report_scipyopt_SLSQP(self):
         self.setup_problem_and_run_driver(om.ScipyOptimizeDriver,
@@ -184,6 +193,17 @@ class TestOptimizationReport(unittest.TestCase):
         opt_report(self.prob)
         self.check_opt_report(expected=expect)
 
+    def test_opt_report_scipyopt_COBYLA_nobounds(self):
+        self.setup_problem_and_run_driver(om.ScipyOptimizeDriver,
+                                          # no bounds on design vars
+                                          cons_lower=0, cons_upper=10.,
+                                          optimizer='COBYLA',
+                                          )
+        expect = {'deriv_calls': None}
+        self.check_opt_result(expected=expect)
+        opt_report(self.prob)
+        self.check_opt_report(expected=expect)
+
     @require_pyoptsparse('SNOPT')
     def test_opt_report_pyoptsparse_snopt(self):
         self.setup_problem_and_run_driver(om.pyOptSparseDriver,
@@ -231,13 +251,14 @@ class TestOptimizationReport(unittest.TestCase):
 
         self.check_opt_result(prob.driver.opt_result, expected={'obj_calls': 0, 'deriv_calls': 0})
 
-        expected_warning_msg = "The optimizer report is not applicable for the DOEDriver Driver " \
+        expected_warning_msg = "The optimizer report is not applicable for Driver type 'DOEDriver', " \
                                "which does not support optimization"
         with assert_warning(DriverWarning, expected_warning_msg):
             opt_report(prob)
         outfilepath = str(pathlib.Path(prob.get_reports_dir()).joinpath(_default_optimizer_report_filename))
         self.assertFalse(os.path.exists(outfilepath))
 
+    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
     def test_opt_report_genetic_algorithm(self):
         self.setup_problem_and_run_driver(om.SimpleGADriver,
                                           vars_lower=-50, vars_upper=50.,
@@ -248,6 +269,7 @@ class TestOptimizationReport(unittest.TestCase):
         opt_report(self.prob)
         self.check_opt_report(expected=expect)
 
+    @unittest.skipUnless(pyDOE2, "requires 'pyDOE2', install openmdao[doe]")
     def test_opt_report_differential_evolution(self):
         prob = om.Problem()
 
